# Test board-specific items on PYBv1.x

import os, pyb

if not "PYBv1." in os.uname().machine:
    print("SKIP")
    raise SystemExit

# test creating UART by id/name
for bus in (1, 2, 3, 4, 5, 6, 7, "XA", "XB", "YA", "YB", "Z"):
    try:
        pyb.UART(bus, 9600)
        print("UART", bus)
    except ValueError:
        print("ValueError", bus)

# test creating SPI by id/name
for bus in (1, 2, 3, "X", "Y", "Z"):
    try:
        pyb.SPI(bus)
        print("SPI", bus)
    except ValueError:
        print("ValueError", bus)

# test creating I2C by id/name
for bus in (2, 3, "X", "Y", "Z"):
    try:
        pyb.I2C(bus)
        print("I2C", bus)
    except ValueError:
        print("ValueError", bus)

# test creating CAN by id/name
for bus in (1, 2, 3, "YA", "YB", "YC"):
    try:
        pyb.CAN(bus, pyb.CAN.LOOPBACK)
        print("CAN", bus)
    except ValueError:
        print("ValueError", bus)



try:
    import uctypes
except ImportError:
    print("SKIP")
    raise SystemExit

S1 = {}
assert uctypes.sizeof(S1) == 0

S2 = {"a": uctypes.UINT8 | 0}
assert uctypes.sizeof(S2) == 1

S3 = {
    "a": uctypes.UINT8 | 0,
    "b": uctypes.UINT8 | 1,
}
assert uctypes.sizeof(S3) == 2

S4 = {
    "a": uctypes.UINT8 | 0,
    "b": uctypes.UINT32 | 4,
    "c": uctypes.UINT8 | 8,
}
assert uctypes.sizeof(S4) == 12

S5 = {
    "a": uctypes.UINT8 | 0,
    "b": uctypes.UINT32 | 4,
    "c": uctypes.UINT8 | 8,
    "d": uctypes.UINT32 | 0,
    "sub": (4, {"b0": uctypes.UINT8 | 0, "b1": uctypes.UINT8 | 1}),
}

assert uctypes.sizeof(S5) == 12

s5 = uctypes.struct(0, S5)
assert uctypes.sizeof(s5) == 12
assert uctypes.sizeof(s5.sub) == 2

S6 = {
    "ptr": (uctypes.PTR | 0, uctypes.UINT8),
}
# As if there're no other arch bitnesses
assert uctypes.sizeof(S6) in (4, 8)

S7 = {
    "arr": (uctypes.ARRAY | 0, uctypes.UINT8 | 5),
}
assert uctypes.sizeof(S7) == 5

S8 = {
    "arr": (uctypes.ARRAY | 0, 3, {"a": uctypes.UINT32 | 0, "b": uctypes.UINT8 | 4}),
}
assert uctypes.sizeof(S8) == 24



"""
The MIT License (MIT)

Copyright (c) 2023 Arduino SA
Copyright (c) 2018 KPN (Jan Bogaerts)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""


from senml.senml_record import SenmlRecord
from senml.senml_base import SenmlBase
import json
import cbor2


class SenmlPackIterator:
    """an iterator to walk over all records in a pack"""

    def __init__(self, list):
        self._list = list
        self._index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self._index < len(self._list):
            res = self._list[self._index]
            self._index += 1
            return res
        else:
            raise StopIteration


class SenmlPack(SenmlBase):
    """
    represents a sneml pack object. This can contain multiple records but also other (child) pack objects.
    When the pack object only contains records, it represents the data of a device.
    If the pack object has child pack objects, then it represents a gateway
    """

    json_mappings = {
        "bn": "bn",
        "bt": "bt",
        "bu": "bu",
        "bv": "bv",
        "bs": "bs",
        "n": "n",
        "u": "u",
        "v": "v",
        "vs": "vs",
        "vb": "vb",
        "vd": "vd",
        "s": "s",
        "t": "t",
        "ut": "ut",
    }

    def __init__(self, name, callback=None):
        """
        initialize the object
        :param name: {string} the name of the pack
        """
        self._data = []
        self.name = name
        self._base_value = None
        self._base_time = None
        self._base_sum = None
        self.base_unit = None
        self._parent = None  # a pack can also be the child of another pack.
        self.actuate = callback  # actuate callback function

    def __iter__(self):
        return SenmlPackIterator(self._data)

    def __enter__(self):
        """
        for supporting the 'with' statement
        :return: self
        """
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        when destroyed in a 'with' statement, make certain that the item is removed from the parent list.
        :return: None
        """
        if self._parent:
            self._parent.remove(self)

    @property
    def base_value(self):
        """
        the base value of the pack.
        :return: a number
        """
        return self._base_value

    @base_value.setter
    def base_value(self, value):
        """
        set the base value.
        :param value: only number allowed
        :return:
        """
        self._check_value_type(value, "base_value")
        self._base_value = value

    @property
    def base_sum(self):
        """
        the base sum of the pack.
        :return: a number
        """
        return self._base_sum

    @base_sum.setter
    def base_sum(self, value):
        """
        set the base value.
        :param value: only number allowed
        :return:
        """
        self._check_value_type(value, "base_sum")
        self._base_sum = value

    @property
    def base_time(self):
        return self._base_time

    @base_time.setter
    def base_time(self, value):
        self._check_value_type(value, "base_time")
        self._base_time = value

    def _check_value_type(self, value, field_name):
        """
        checks if the type of value is allowed for senml
        :return: None, raisee exception if not ok.
        """
        if value is not None:
            if not (isinstance(value, int) or isinstance(value, float)):
                raise Exception("invalid type for " + field_name + ", only numbers allowed")

    def from_json(self, data):
        """
        parse a json string and convert it to a senml pack structure
        :param data: a string containing json data.
        :return: None, will r
        """
        records = json.loads(data)  # load the raw senml data
        self._process_incomming_data(records, SenmlPack.json_mappings)

    def _process_incomming_data(self, records, naming_map):
        """
        generic processor for incomming data (actuators.
        :param records: the list of raw senml data, parsed from a json or cbor structure
        :param naming_map: translates cbor to json field names (when needed).
        :return: None
        """
        cur_pack_el = self
        new_pack = False
        for item in records:
            if naming_map["bn"] in item:  # ref to a pack element, either this or a child pack.
                if item[naming_map["bn"]] != self.name:
                    pack_el = [x for x in self._data if x.name == item[naming_map["bn"]]]
                else:
                    pack_el = [self]
                if len(pack_el) > 0:
                    cur_pack_el = pack_el[0]
                    new_pack = False
                else:
                    device = SenmlPack(item[naming_map["bn"]])
                    self._data.append(device)
                    cur_pack_el = device
                    new_pack = True

                if (
                    naming_map["bv"] in item
                ):  # need to copy the base value assigned to the pack element so we can do proper conversion for actuators.
                    cur_pack_el.base_value = item[naming_map["bv"]]

                rec_el = [x for x in cur_pack_el._data if x.name == item[naming_map["n"]]]
                if len(rec_el) > 0:
                    rec_el[0].do_actuate(item, naming_map)
                elif new_pack:
                    self.do_actuate(item, naming_map, cur_pack_el)
                else:
                    cur_pack_el.do_actuate(item, naming_map)
            else:
                rec_el = [x for x in self._data if x.name == item[naming_map["n"]]]
                if len(rec_el) > 0:
                    rec_el[0].do_actuate(item, naming_map)
                elif new_pack:
                    self.do_actuate(item, naming_map, cur_pack_el)
                else:
                    cur_pack_el.do_actuate(item, naming_map)

    def do_actuate(self, raw, naming_map, device=None):
        """
        called while parsing incoming data for a record that is not yet part of this pack object.
        adds a new record and raises the actuate callback of the pack with the newly created record as argument
        :param naming_map:
        :param device: optional: if the device was not found
        :param raw: the raw record definition, as found in the json structure. this still has invalid labels.
        :return: None
        """
        rec = SenmlRecord(raw[naming_map["n"]])
        if device:
            device.add(rec)
            rec._from_raw(raw, naming_map)
            if self.actuate:
                self.actuate(rec, device=device)
        else:
            self.add(rec)
            rec._from_raw(raw, naming_map)
            if self.actuate:
                self.actuate(rec, device=None)

    def to_json(self):
        """
        render the content of this object to a string.
        :return: a string representing the senml pack object
        """
        converted = []
        self._build_rec_dict(SenmlPack.json_mappings, converted)
        return json.dumps(converted)

    def _build_rec_dict(self, naming_map, appendTo):
        """
        converts the object to a senml object with the proper naming in place.
        This can be recursive: a pack can contain other packs.
        :param naming_map: a dictionary used to pick the correct field names for either senml json or senml cbor
        :return:
        """
        internalList = []
        for item in self._data:
            item._build_rec_dict(naming_map, internalList)
        if len(internalList) > 0:
            first_rec = internalList[0]
        else:
            first_rec = {}
            internalList.append(first_rec)

        if self.name:
            first_rec[naming_map["bn"]] = self.name
        if self.base_value:
            first_rec[naming_map["bv"]] = self.base_value
        if self.base_unit:
            first_rec[naming_map["bu"]] = self.base_unit
        if self.base_sum:
            first_rec[naming_map["bs"]] = self.base_sum
        if self.base_time:
            first_rec[naming_map["bt"]] = self.base_time
        appendTo.extend(internalList)

    def from_cbor(self, data):
        """
        parse a cbor data byte array to a senml pack structure.
        :param data: a byte array.
        :return: None
        """
        records = cbor2.loads(data)  # load the raw senml data
        naming_map = {
            "bn": -2,
            "bt": -3,
            "bu": -4,
            "bv": -5,
            "bs": -16,
            "n": 0,
            "u": 1,
            "v": 2,
            "vs": 3,
            "vb": 4,
            "vd": 8,
            "s": 5,
            "t": 6,
            "ut": 7,
        }
        self._process_incomming_data(records, naming_map)

    def to_cbor(self):
        """
        render the content of this object to a cbor byte array
        :return: a byte array
        """
        naming_map = {
            "bn": -2,
            "bt": -3,
            "bu": -4,
            "bv": -5,
            "bs": -16,
            "n": 0,
            "u": 1,
            "v": 2,
            "vs": 3,
            "vb": 4,
            "vd": 8,
            "s": 5,
            "t": 6,
            "ut": 7,
        }
        converted = []
        self._build_rec_dict(naming_map, converted)
        return cbor2.dumps(converted)

    def add(self, item):
        """
        adds the item to the list of records
        :param item: {SenmlRecord} the item that needs to be added to the pack
        :return: None
        """
        if not (isinstance(item, SenmlBase)):
            raise Exception("invalid type of param, SenmlRecord or SenmlPack expected")
        if item._parent is not None:
            raise Exception("item is already part of a pack")

        self._data.append(item)
        item._parent = self

    def remove(self, item):
        """
        removes the item from the list of records
        :param item: {SenmlRecord} the item that needs to be removed
        :return: None
        """
        if not (isinstance(item, SenmlBase)):
            raise Exception("invalid type of param, SenmlRecord or SenmlPack expected")
        if not item._parent == self:
            raise Exception("item is not part of this pack")

        self._data.remove(item)
        item._parent = None

    def clear(self):
        """
        clear the list of the pack
        :return: None
        """
        for item in self._data:
            item._parent = None
        self._data = []



# test builtin range type

# print
print(range(4))

# bool
print(bool(range(0)))
print(bool(range(10)))

# len
print(len(range(0)))
print(len(range(4)))
print(len(range(1, 4)))
print(len(range(1, 4, 2)))
print(len(range(1, 4, -1)))
print(len(range(4, 1, -1)))
print(len(range(4, 1, -2)))

# subscr
print(range(4)[0])
print(range(4)[1])
print(range(4)[-1])

# slice
print(range(4)[0:])
print(range(4)[1:])
print(range(4)[1:2])
print(range(4)[1:3])
print(range(4)[1::2])
print(range(4)[1:-2:2])
print(range(1, 4)[:])
print(range(1, 4)[0:])
print(range(1, 4)[1:])
print(range(1, 4)[:-1])
print(range(7, -2, -4)[:])
print(range(1, 100, 5)[5:15:3])
print(range(1, 100, 5)[15:5:-3])
print(range(100, 1, -5)[5:15:3])
print(range(100, 1, -5)[15:5:-3])

# for this case uPy gives a different stop value but the listed elements are still correct
print(list(range(7, -2, -4)[2:-2:]))

# zero step
try:
    range(1, 2, 0)
except ValueError:
    print("ValueError")

# bad unary op
try:
    -range(1)
except TypeError:
    print("TypeError")

# bad subscription (can't store)
try:
    range(1)[0] = 1
except TypeError:
    print("TypeError")



# Test that locks work when cancelling multiple waiters on the lock

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


async def task(i, lock, lock_flag):
    print("task", i, "start")
    try:
        await lock.acquire()
    except asyncio.CancelledError:
        print("task", i, "cancel")
        return
    print("task", i, "lock_flag", lock_flag[0])
    lock_flag[0] = True
    await asyncio.sleep(0)
    lock.release()
    lock_flag[0] = False
    print("task", i, "done")


async def main():
    # Create a lock and acquire it so the tasks below must wait
    lock = asyncio.Lock()
    await lock.acquire()
    lock_flag = [True]

    # Create 4 tasks and let them all run
    t0 = asyncio.create_task(task(0, lock, lock_flag))
    t1 = asyncio.create_task(task(1, lock, lock_flag))
    t2 = asyncio.create_task(task(2, lock, lock_flag))
    t3 = asyncio.create_task(task(3, lock, lock_flag))
    await asyncio.sleep(0)

    # Cancel 2 of the tasks (which are waiting on the lock) and release the lock
    t1.cancel()
    t2.cancel()
    lock.release()
    lock_flag[0] = False

    # Let the tasks run to completion
    for _ in range(4):
        await asyncio.sleep(0)

    # The locke should be unlocked
    print(lock.locked())


asyncio.run(main())



#!/usr/bin/env python
"""Generates the pins file for the CC3200."""

# Do not use this as a reference for new ports. See tools/boardgen.py and e.g.
# ports/stm32/boards/make-pins.py.

from __future__ import print_function

import argparse
import sys
import csv


SUPPORTED_AFS = {
    "UART": ("TX", "RX", "RTS", "CTS"),
    "SPI": ("CLK", "MOSI", "MISO", "CS0"),
    #'I2S': ('CLK', 'FS', 'DAT0', 'DAT1'),
    "I2C": ("SDA", "SCL"),
    "TIM": ("PWM"),
    "SD": ("CLK", "CMD", "DAT0"),
    "ADC": ("CH0", "CH1", "CH2", "CH3"),
}


def parse_port_pin(name_str):
    """Parses a string and returns a (port, gpio_bit) tuple."""
    if len(name_str) < 3:
        raise ValueError("Expecting pin name to be at least 3 characters")
    if name_str[:2] != "GP":
        raise ValueError("Expecting pin name to start with GP")
    if not name_str[2:].isdigit():
        raise ValueError("Expecting numeric GPIO number")
    port = int(int(name_str[2:]) / 8)
    gpio_bit = 1 << int(int(name_str[2:]) % 8)
    return (port, gpio_bit)


class AF:
    """Holds the description of an alternate function"""

    def __init__(self, name, idx, fn, unit, type):
        self.name = name
        self.idx = idx
        if self.idx > 15:
            self.idx = -1
        self.fn = fn
        self.unit = unit
        self.type = type

    def print(self, out_source):
        print(
            "    AF({:16s}, {:4d}, {:8s}, {:4d}, {:8s}),    // {}".format(
                self.name, self.idx, self.fn, self.unit, self.type, self.name
            ),
            file=out_source,
        )


class Pin:
    """Holds the information associated with a pin."""

    def __init__(self, name, port, gpio_bit, pin_num):
        self.name = name
        self.port = port
        self.gpio_bit = gpio_bit
        self.pin_num = pin_num
        self.board_pin = False
        self.afs = []

    def add_af(self, af):
        self.afs.append(af)

    def print(self, out_source):
        print("// {}".format(self.name), file=out_source)
        if len(self.afs):
            print("const pin_af_t pin_{}_af[] = {{".format(self.name), file=out_source)
            for af in self.afs:
                af.print(out_source)
            print("};", file=out_source)
            print(
                "pin_obj_t pin_{:4s} = PIN({:6s}, {:1d}, {:3d}, {:2d}, pin_{}_af, {});\n".format(
                    self.name,
                    self.name,
                    self.port,
                    self.gpio_bit,
                    self.pin_num,
                    self.name,
                    len(self.afs),
                ),
                file=out_source,
            )
        else:
            print(
                "pin_obj_t pin_{:4s} = PIN({:6s}, {:1d}, {:3d}, {:2d}, NULL, 0);\n".format(
                    self.name, self.name, self.port, self.gpio_bit, self.pin_num
                ),
                file=out_source,
            )

    def print_header(self, out_header):
        print("extern pin_obj_t pin_{:s};".format(self.name), file=out_header)


class Pins:
    def __init__(self):
        self.board_pins = []  # list of pin objects

    def find_pin(self, port, gpio_bit):
        for pin in self.board_pins:
            if pin.port == port and pin.gpio_bit == gpio_bit:
                return pin

    def find_pin_by_num(self, pin_num):
        for pin in self.board_pins:
            if pin.pin_num == pin_num:
                return pin

    def find_pin_by_name(self, name):
        for pin in self.board_pins:
            if pin.name == name:
                return pin

    def parse_af_file(self, filename, pin_col, pinname_col, af_start_col):
        with open(filename, "r") as csvfile:
            rows = csv.reader(csvfile)
            for row in rows:
                try:
                    (port_num, gpio_bit) = parse_port_pin(row[pinname_col])
                except:
                    continue
                if not row[pin_col].isdigit():
                    raise ValueError(
                        "Invalid pin number {:s} in row {:s}".format(row[pin_col], row)
                    )
                # Pin numbers must start from 0 when used with the TI API
                pin_num = int(row[pin_col]) - 1
                pin = Pin(row[pinname_col], port_num, gpio_bit, pin_num)
                self.board_pins.append(pin)
                af_idx = 0
                for af in row[af_start_col:]:
                    af_splitted = af.split("_")
                    fn_name = af_splitted[0].rstrip("0123456789")
                    if fn_name in SUPPORTED_AFS:
                        type_name = af_splitted[1]
                        if type_name in SUPPORTED_AFS[fn_name]:
                            unit_idx = af_splitted[0][-1]
                            pin.add_af(AF(af, af_idx, fn_name, int(unit_idx), type_name))
                    af_idx += 1

    def parse_board_file(self, filename, cpu_pin_col):
        with open(filename, "r") as csvfile:
            rows = csv.reader(csvfile)
            for row in rows:
                # Pin numbers must start from 0 when used with the TI API
                if row[cpu_pin_col].isdigit():
                    pin = self.find_pin_by_num(int(row[cpu_pin_col]) - 1)
                else:
                    pin = self.find_pin_by_name(row[cpu_pin_col])
                if pin:
                    pin.board_pin = True

    def print_named(self, label, pins, out_source):
        print("", file=out_source)
        print(
            "static const mp_rom_map_elem_t pin_{:s}_pins_locals_dict_table[] = {{".format(label),
            file=out_source,
        )
        for pin in pins:
            if pin.board_pin:
                print(
                    "    {{ MP_ROM_QSTR(MP_QSTR_{:6s}), MP_ROM_PTR(&pin_{:6s}) }},".format(
                        pin.name, pin.name
                    ),
                    file=out_source,
                )
        print("};", file=out_source)
        print(
            "MP_DEFINE_CONST_DICT(pin_{:s}_pins_locals_dict, pin_{:s}_pins_locals_dict_table);".format(
                label, label
            ),
            file=out_source,
        )

    def print(self, out_source):
        for pin in self.board_pins:
            if pin.board_pin:
                pin.print(out_source)
        self.print_named("board", self.board_pins, out_source)
        print("", file=out_source)

    def print_header(self, out_header):
        for pin in self.board_pins:
            if pin.board_pin:
                pin.print_header(out_header)


def main():
    parser = argparse.ArgumentParser(description="Generate board specific pin file")
    parser.add_argument("--board-csv")
    parser.add_argument("--af-csv")
    parser.add_argument("--prefix")
    parser.add_argument("--output-source")
    parser.add_argument("--output-header")
    args = parser.parse_args()

    pins = Pins()

    with open(args.output_source, "w") as out_source:
        print("// This file was automatically generated by make-pins.py", file=out_source)
        print("//", file=out_source)
        if args.af_csv:
            print("// --af {:s}".format(args.af_csv), file=out_source)
            pins.parse_af_file(args.af_csv, 0, 1, 3)

        if args.board_csv:
            print("// --board {:s}".format(args.board_csv), file=out_source)
            pins.parse_board_file(args.board_csv, 1)

        if args.prefix:
            print("// --prefix {:s}".format(args.prefix), file=out_source)
            print("", file=out_source)
            with open(args.prefix, "r") as prefix_file:
                print(prefix_file.read(), file=out_source)
        pins.print(out_source)

    with open(args.output_header, "w") as out_header:
        pins.print_header(out_header)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# This tool converts binary resource files passed on the command line
# into a Python source file containing data from these files, which can
# be accessed using standard pkg_resources.resource_stream() function
# from micropython-lib:
# https://github.com/micropython/micropython-lib/tree/master/pkg_resources
#
import sys

print("R = {")

for fname in sys.argv[1:]:
    with open(fname, "rb") as f:
        b = f.read()
        print("%r: %r," % (fname, b))

print("}")



import jwt
from time import time

secret_key = "top-secret!"

token = jwt.encode({"user": "joe"}, secret_key, algorithm="HS256")
print(token)
decoded = jwt.decode(token, secret_key, algorithms=["HS256"])
if decoded != {"user": "joe"}:
    raise Exception("Invalid decoded JWT")
else:
    print("Encode/decode test: OK")

try:
    decoded = jwt.decode(token, "wrong-secret", algorithms=["HS256"])
except jwt.exceptions.InvalidSignatureError:
    print("Invalid signature test: OK")
else:
    raise Exception("Invalid JWT should have failed decoding")

token = jwt.encode({"user": "joe", "exp": time() - 1}, secret_key)
print(token)
try:
    decoded = jwt.decode(token, secret_key, algorithms=["HS256"])
except jwt.exceptions.ExpiredSignatureError:
    print("Expired token test: OK")
else:
    raise Exception("Expired JWT should have failed decoding")



# test bytearray with its reuse of byte functions

print(bytearray(b"hello world").find(b"ll"))
print(bytearray(b"hello\x00world").rfind(b"l"))

print(bytearray(b"abc  efg ").strip(b"g a"))
print(bytearray(b"   spacious   ").lstrip())
print(bytearray(b"www.example.com").lstrip(b"cmowz."))
print(bytearray(b"   spacious   ").rstrip())
print(bytearray(b"mississippi").rstrip(b"ipz"))

print(bytearray(b"abc").split(b"a"))
print(bytearray(b"abcabc").rsplit(b"bc"))

print(bytearray(b"asdfasdf").replace(b"a", b"b"))

print("00\x0000".index("0", 0))
print("00\x0000".index("0", 3))
print("00\x0000".rindex("0", 0))
print("00\x0000".rindex("0", 3))

print(bytearray(b"foobar").endswith(b"bar"))
print(bytearray(b"1foo").startswith(b"foo", 1))

print(bytearray(b" T E \x00 S T").lower())
print(bytearray(b" te \x00 st").upper())

print(bytearray(b" \t\n\r\v\f").isspace())
print(bytearray(b"this ").isalpha())
print(bytearray(b"0123456789").isdigit())
print(bytearray(b"AB").isupper())
print(bytearray(b"cheese-cake").islower())

print(bytearray(b",").join((bytearray(b"abc"), bytearray(b"def"))))
print(type(bytearray(b",").join((b"a", b"b", b"c"))))



"""
IMU module for Arduino Nano BLE 33 SENSE (REV1 and REV2).

Example usage:

import time
import imu
from machine import Pin, I2C

bus = I2C(1, scl=Pin(15), sda=Pin(14))
imu = imu.IMU(bus)

while (True):
    print('Accelerometer: x:{:>8.3f} y:{:>8.3f} z:{:>8.3f}'.format(*imu.accel()))
    print('Gyroscope:     x:{:>8.3f} y:{:>8.3f} z:{:>8.3f}'.format(*imu.gyro()))
    print('Magnetometer:  x:{:>8.3f} y:{:>8.3f} z:{:>8.3f}'.format(*imu.magnet()))
    print("")
    time.sleep_ms(100)
"""

import time


class IMU:
    def __init__(self, bus):
        """Initializes Gyro, Accelerometer and Magnetometer using default values."""
        if 0x68 in bus.scan():
            from bmm150 import BMM150
            from bmi270 import BMI270

            magnet = BMM150(bus)
            self.imu = BMI270(bus, bmm_magnet=magnet)
        else:
            from lsm9ds1 import LSM9DS1

            self.imu = LSM9DS1(bus)

    def gyro(self):
        return self.imu.gyro()

    def accel(self):
        return self.imu.accel()

    def magnet(self):
        return self.imu.magnet()



# This tests extended (MicroPython-specific) form of write:
# write(buf, len) and write(buf, offset, len)
import io

try:
    io.BytesIO
except AttributeError:
    print('SKIP')
    raise SystemExit

buf = io.BytesIO()

buf.write(b"foo", 2)
print(buf.getvalue())

buf.write(b"foo", 100)
print(buf.getvalue())

buf.write(b"foobar", 1, 3)
print(buf.getvalue())

buf.write(b"foobar", 1, 100)
print(buf.getvalue())

buf.write(b"foobar", 100, 100)
print(buf.getvalue())



import io
import os
import sys

try:
    import traceback
except ImportError:
    traceback = None


class SkipTest(Exception):
    pass


class AssertRaisesContext:
    def __init__(self, exc):
        self.expected = exc

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.exception = exc_value
        if exc_type is None:
            assert False, "%r not raised" % self.expected
        if issubclass(exc_type, self.expected):
            # store exception for later retrieval
            self.exception = exc_value
            return True
        return False


# These are used to provide required context to things like subTest
__current_test__ = None
__test_result__ = None


class SubtestContext:
    def __init__(self, msg=None, params=None):
        self.msg = msg
        self.params = params

    def __enter__(self):
        pass

    def __exit__(self, *exc_info):
        if exc_info[0] is not None:
            # Exception raised
            global __test_result__, __current_test__
            test_details = __current_test__
            if self.msg:
                test_details += (f" [{self.msg}]",)
            if self.params:
                detail = ", ".join(f"{k}={v}" for k, v in self.params.items())
                test_details += (f" ({detail})",)

            _handle_test_exception(test_details, __test_result__, exc_info, False)
        # Suppress the exception as we've captured it above
        return True


class NullContext:
    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        pass


class TestCase:
    def __init__(self):
        pass

    def addCleanup(self, func, *args, **kwargs):
        if not hasattr(self, "_cleanups"):
            self._cleanups = []
        self._cleanups.append((func, args, kwargs))

    def doCleanups(self):
        if hasattr(self, "_cleanups"):
            while self._cleanups:
                func, args, kwargs = self._cleanups.pop()
                func(*args, **kwargs)

    def subTest(self, msg=None, **params):
        return SubtestContext(msg=msg, params=params)

    def skipTest(self, reason):
        raise SkipTest(reason)

    def fail(self, msg=""):
        assert False, msg

    def assertEqual(self, x, y, msg=""):
        if not msg:
            msg = "%r vs (expected) %r" % (x, y)
        assert x == y, msg

    def assertNotEqual(self, x, y, msg=""):
        if not msg:
            msg = "%r not expected to be equal %r" % (x, y)
        assert x != y, msg

    def assertLessEqual(self, x, y, msg=None):
        if msg is None:
            msg = "%r is expected to be <= %r" % (x, y)
        assert x <= y, msg

    def assertGreaterEqual(self, x, y, msg=None):
        if msg is None:
            msg = "%r is expected to be >= %r" % (x, y)
        assert x >= y, msg

    def assertAlmostEqual(self, x, y, places=None, msg="", delta=None):
        if x == y:
            return
        if delta is not None and places is not None:
            raise TypeError("specify delta or places not both")

        if delta is not None:
            if abs(x - y) <= delta:
                return
            if not msg:
                msg = "%r != %r within %r delta" % (x, y, delta)
        else:
            if places is None:
                places = 7
            if round(abs(y - x), places) == 0:
                return
            if not msg:
                msg = "%r != %r within %r places" % (x, y, places)

        assert False, msg

    def assertNotAlmostEqual(self, x, y, places=None, msg="", delta=None):
        if delta is not None and places is not None:
            raise TypeError("specify delta or places not both")

        if delta is not None:
            if not (x == y) and abs(x - y) > delta:
                return
            if not msg:
                msg = "%r == %r within %r delta" % (x, y, delta)
        else:
            if places is None:
                places = 7
            if not (x == y) and round(abs(y - x), places) != 0:
                return
            if not msg:
                msg = "%r == %r within %r places" % (x, y, places)

        assert False, msg

    def assertIs(self, x, y, msg=""):
        if not msg:
            msg = "%r is not %r" % (x, y)
        assert x is y, msg

    def assertIsNot(self, x, y, msg=""):
        if not msg:
            msg = "%r is %r" % (x, y)
        assert x is not y, msg

    def assertIsNone(self, x, msg=""):
        if not msg:
            msg = "%r is not None" % x
        assert x is None, msg

    def assertIsNotNone(self, x, msg=""):
        if not msg:
            msg = "%r is None" % x
        assert x is not None, msg

    def assertTrue(self, x, msg=""):
        if not msg:
            msg = "Expected %r to be True" % x
        assert x, msg

    def assertFalse(self, x, msg=""):
        if not msg:
            msg = "Expected %r to be False" % x
        assert not x, msg

    def assertIn(self, x, y, msg=""):
        if not msg:
            msg = "Expected %r to be in %r" % (x, y)
        assert x in y, msg

    def assertIsInstance(self, x, y, msg=""):
        assert isinstance(x, y), msg

    def assertRaises(self, exc, func=None, *args, **kwargs):
        if func is None:
            return AssertRaisesContext(exc)

        try:
            func(*args, **kwargs)
        except Exception as e:
            if isinstance(e, exc):
                return
            raise e

        assert False, "%r not raised" % exc

    def assertWarns(self, warn):
        return NullContext()


def skip(msg):
    def _decor(fun):
        # We just replace original fun with _inner
        def _inner(self):
            raise SkipTest(msg)

        return _inner

    return _decor


def skipIf(cond, msg):
    if not cond:
        return lambda x: x
    return skip(msg)


def skipUnless(cond, msg):
    if cond:
        return lambda x: x
    return skip(msg)


def expectedFailure(test):
    def test_exp_fail(*args, **kwargs):
        try:
            test(*args, **kwargs)
        except:
            pass
        else:
            assert False, "unexpected success"

    return test_exp_fail


class TestSuite:
    def __init__(self, name=""):
        self._tests = []
        self.name = name

    def addTest(self, cls):
        self._tests.append(cls)

    def run(self, result):
        for c in self._tests:
            _run_suite(c, result, self.name)
        return result

    def _load_module(self, mod):
        for tn in dir(mod):
            c = getattr(mod, tn)
            if isinstance(c, object) and isinstance(c, type) and issubclass(c, TestCase):
                self.addTest(c)
            elif tn.startswith("test") and callable(c):
                self.addTest(c)


class TestRunner:
    def run(self, suite: TestSuite):
        res = TestResult()
        suite.run(res)

        res.printErrors()
        print("----------------------------------------------------------------------")
        print("Ran %d tests\n" % res.testsRun)
        if res.failuresNum > 0 or res.errorsNum > 0:
            print("FAILED (failures=%d, errors=%d)" % (res.failuresNum, res.errorsNum))
        else:
            msg = "OK"
            if res.skippedNum > 0:
                msg += " (skipped=%d)" % res.skippedNum
            print(msg)

        return res


TextTestRunner = TestRunner


class TestResult:
    def __init__(self):
        self.errorsNum = 0
        self.failuresNum = 0
        self.skippedNum = 0
        self.testsRun = 0
        self.errors = []
        self.failures = []
        self.skipped = []
        self._newFailures = 0

    def wasSuccessful(self):
        return self.errorsNum == 0 and self.failuresNum == 0

    def printErrors(self):
        if self.errors or self.failures:
            print()
            self.printErrorList(self.errors)
            self.printErrorList(self.failures)

    def printErrorList(self, lst):
        sep = "----------------------------------------------------------------------"
        for c, e in lst:
            detail = " ".join((str(i) for i in c))
            print("======================================================================")
            print(f"FAIL: {detail}")
            print(sep)
            print(e)

    def __repr__(self):
        # Format is compatible with CPython.
        return "<unittest.result.TestResult run=%d errors=%d failures=%d>" % (
            self.testsRun,
            self.errorsNum,
            self.failuresNum,
        )

    def __add__(self, other):
        self.errorsNum += other.errorsNum
        self.failuresNum += other.failuresNum
        self.skippedNum += other.skippedNum
        self.testsRun += other.testsRun
        self.errors.extend(other.errors)
        self.failures.extend(other.failures)
        self.skipped.extend(other.skipped)
        return self


def _capture_exc(exc, exc_traceback):
    buf = io.StringIO()
    if hasattr(sys, "print_exception"):
        sys.print_exception(exc, buf)
    elif traceback is not None:
        traceback.print_exception(None, exc, exc_traceback, file=buf)
    return buf.getvalue()


def _handle_test_exception(
    current_test: tuple, test_result: TestResult, exc_info: tuple, verbose=True
):
    exc = exc_info[1]
    traceback = exc_info[2]
    ex_str = _capture_exc(exc, traceback)
    if isinstance(exc, SkipTest):
        reason = exc.args[0]
        test_result.skippedNum += 1
        test_result.skipped.append((current_test, reason))
        print(" skipped:", reason)
        return
    elif isinstance(exc, AssertionError):
        test_result.failuresNum += 1
        test_result.failures.append((current_test, ex_str))
        if verbose:
            print(" FAIL")
    else:
        test_result.errorsNum += 1
        test_result.errors.append((current_test, ex_str))
        if verbose:
            print(" ERROR")
    test_result._newFailures += 1


def _run_suite(c, test_result: TestResult, suite_name=""):
    if isinstance(c, TestSuite):
        c.run(test_result)
        return

    if isinstance(c, type):
        o = c()
    else:
        o = c
    set_up_class = getattr(o, "setUpClass", lambda: None)
    tear_down_class = getattr(o, "tearDownClass", lambda: None)
    set_up = getattr(o, "setUp", lambda: None)
    tear_down = getattr(o, "tearDown", lambda: None)
    exceptions = []
    try:
        suite_name += "." + c.__qualname__
    except AttributeError:
        pass

    def run_one(test_function):
        global __test_result__, __current_test__
        print("%s (%s) ..." % (name, suite_name), end="")
        set_up()
        __test_result__ = test_result
        test_container = f"({suite_name})"
        __current_test__ = (name, test_container)
        try:
            test_result._newFailures = 0
            test_result.testsRun += 1
            test_function()
            # No exception occurred, test passed
            if test_result._newFailures:
                print(" FAIL")
            else:
                print(" ok")
        except Exception as ex:
            _handle_test_exception(
                current_test=(name, c), test_result=test_result, exc_info=(type(ex), ex, None)
            )
            # Uncomment to investigate failure in detail
            # raise ex
        finally:
            __test_result__ = None
            __current_test__ = None
            tear_down()
            try:
                o.doCleanups()
            except AttributeError:
                pass

    set_up_class()
    try:
        if hasattr(o, "runTest"):
            name = str(o)
            run_one(o.runTest)
            return

        for name in dir(o):
            if name.startswith("test"):
                m = getattr(o, name)
                if not callable(m):
                    continue
                run_one(m)

        if callable(o):
            name = o.__name__
            run_one(o)
    finally:
        tear_down_class()

    return exceptions


# This supports either:
#
# >>> import mytest
# >>> unitttest.main(mytest)
#
# >>> unittest.main("mytest")
#
# Or, a script that ends with:
# if __name__ == "__main__":
#     unittest.main()
# e.g. run via `mpremote run mytest.py`
def main(module="__main__", testRunner=None):
    if testRunner is None:
        testRunner = TestRunner()
    elif isinstance(testRunner, type):
        testRunner = testRunner()

    if isinstance(module, str):
        module = __import__(module)
    suite = TestSuite(module.__name__)
    suite._load_module(module)
    return testRunner.run(suite)



# MH-Z19 CO2 sensor driver for MicroPython.
# MIT license; Copyright (c) 2018 Fredrik Strupe

import machine
import utime


class TimeoutError(Exception):
    pass


class MHZ19:
    """MH-Z19 CO2 sensor driver"""

    def __init__(self, pin, max_value=5000):
        """
        Args:
            pin: the pin that the PWM pin on the MH-Z19 is connected to.
            max_value: upper bound of measuring range. usually 2000 or 5000.
        """
        self.pin = pin
        self.max_value = max_value

    def _wait_on_condition(self, cond, timeout=5000):
        start = utime.ticks_ms()
        while not cond():
            if utime.ticks_diff(utime.ticks_ms(), start) > timeout:
                raise TimeoutError

    def pwm_read(self):
        """Read CO2 value via PWM pin.

        Reading usually takes 1-2 seconds.

        Returns:
            CO2 value in ppm (parts per million), with an accuracy of
            ±(50 + result * 0.05) ppm.
        Raises:
            TimeoutError: if the reading takes more than 5 seconds.
        """
        # Wait until a new cycle starts
        self._wait_on_condition(lambda: self.pin.value() == 0)

        # Measure high and low duration during cycle
        t_h = machine.time_pulse_us(self.pin, 1, 1500000)
        t_l = machine.time_pulse_us(self.pin, 0, 1500000)

        return self.max_value * (t_h - 2000) / (t_h + t_l - 4000)



# test I2C errors, with polling (disabled irqs) and DMA

import pyb
from pyb import I2C

if not hasattr(pyb, "Accel"):
    print("SKIP")
    raise SystemExit

# init accelerometer
pyb.Accel()

# get I2C bus
i2c = I2C(1, I2C.CONTROLLER, dma=True)

# test polling mem_read
pyb.disable_irq()
i2c.mem_read(1, 76, 0x0A)  # should succeed
pyb.enable_irq()
try:
    pyb.disable_irq()
    i2c.mem_read(1, 77, 0x0A)  # should fail
except OSError as e:
    pyb.enable_irq()
    print(repr(e))
i2c.mem_read(1, 76, 0x0A)  # should succeed

# test polling mem_write
pyb.disable_irq()
i2c.mem_write(1, 76, 0x0A)  # should succeed
pyb.enable_irq()
try:
    pyb.disable_irq()
    i2c.mem_write(1, 77, 0x0A)  # should fail
except OSError as e:
    pyb.enable_irq()
    print(repr(e))
i2c.mem_write(1, 76, 0x0A)  # should succeed

# test DMA mem_read
i2c.mem_read(1, 76, 0x0A)  # should succeed
try:
    i2c.mem_read(1, 77, 0x0A)  # should fail
except OSError as e:
    print(repr(e))
i2c.mem_read(1, 76, 0x0A)  # should succeed

# test DMA mem_write
i2c.mem_write(1, 76, 0x0A)  # should succeed
try:
    i2c.mem_write(1, 77, 0x0A)  # should fail
except OSError as e:
    print(repr(e))
i2c.mem_write(1, 76, 0x0A)  # should succeed



# test that the following do not use the heap:
# - basic scheduling of tasks
# - asyncio.sleep_ms
# - StreamWriter.write, stream is blocked and data to write is a bytes object
# - StreamWriter.write, when stream is not blocked

import micropython

# strict stackless builds can't call functions without allocating a frame on the heap
try:
    # force bytecode (in case we're running with emit=native) and verify
    # that bytecode-calling-bytecode doesn't allocate
    @micropython.bytecode
    def f(x):
        x and f(x - 1)

    micropython.heap_lock()
    f(1)
    micropython.heap_unlock()
except RuntimeError:
    # RuntimeError (max recursion depth) not MemoryError because effectively
    # the recursion depth is at the limit while the heap is locked with
    # stackless
    print("SKIP")
    raise SystemExit

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


class TestStream:
    def __init__(self, blocked):
        self.blocked = blocked

    def write(self, data):
        print("TestStream.write", data)
        if self.blocked:
            return None
        return len(data)


async def task(id, n, t):
    for i in range(n):
        print(id, i)
        await asyncio.sleep_ms(t)


async def main():
    t1 = asyncio.create_task(task(1, 4, 100))
    t2 = asyncio.create_task(task(2, 2, 250))

    # test scheduling tasks, and calling sleep_ms
    micropython.heap_lock()
    print("start")
    await asyncio.sleep_ms(5)
    print("sleep")
    await asyncio.sleep_ms(350)
    print("finish")
    micropython.heap_unlock()

    # test writing to a stream, when the underlying stream is blocked
    s = asyncio.StreamWriter(TestStream(True), None)
    micropython.heap_lock()
    s.write(b"12")
    micropython.heap_unlock()

    # test writing to a stream, when the underlying stream is not blocked
    buf = bytearray(b"56")
    s = asyncio.StreamWriter(TestStream(False), None)
    micropython.heap_lock()
    s.write(b"34")
    s.write(buf)
    micropython.heap_unlock()


asyncio.run(main())



try:
    import binascii
except ImportError:
    print("SKIP")
    raise SystemExit

for x in (
    b"\x00\x01\x02\x03\x04\x05\x06\x07",
    b"\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
    b"\x7f\x80\xff",
    b"1234ABCDabcd",
):
    print(binascii.hexlify(x))

# Two-argument version (now supported in CPython)
print(binascii.hexlify(b"123", ":"))

# zero length buffer
print(binascii.hexlify(b"", b":"))



# tests basics of bound methods

# uPy and CPython differ when printing a bound method, so just print the type
print(type(repr([].append)))


class A:
    def f(self):
        return 0

    def g(self, a):
        return a

    def h(self, a, b, c, d, e, f):
        return a + b + c + d + e + f


# bound method with no extra args
m = A().f
print(m())

# bound method with 1 extra arg
m = A().g
print(m(1))

# bound method with lots of extra args
m = A().h
print(m(1, 2, 3, 4, 5, 6))

# can't assign attributes to a bound method
try:
    A().f.x = 1
except AttributeError:
    print("AttributeError")

print("--------")

# bound method comparison with same object
a = A()
m1 = a.f
m2 = a.f
print(m1 == a.f)  # should result in True
print(m2 == a.f)  # should result in True
print(m1 == m2)  # should result in True
print(m1 != a.f)  # should result in False

# bound method comparison with different objects
a1 = A()
a2 = A()
m1 = a1.f
m2 = a2.f
print(m1 == a2.f)  # should result in False
print(m2 == a1.f)  # should result in False
print(m1 != a2.f)  # should result in True

# bound method comparison with non-bound-method objects
print(A().f == None)  # should result in False
print(A().f != None)  # should result in True
print(None == A().f)  # should result in False
print(None != A().f)  # should result in True

print("--------")

# bound method hashing
a = A()
m1 = a.f
m2 = a.f
print(hash(m1) == hash(a.f))  # should result in True
print(hash(m2) == hash(a.f))  # should result in True
print(hash(m1) == hash(m2))  # should result in True
print(hash(m1) != hash(a.g))  # should result in True

# bound method hashing with different objects
a2 = A()
m2 = a2.f
print(hash(m1) == hash(a2.f))  # should result in False



# Tests the functions imported from math

try:
    from math import *
except ImportError:
    print("SKIP")
    raise SystemExit

test_values = [-100.0, -1.23456, -1, -0.5, 0.0, 0.5, 1.23456, 100.0]
test_values_small = [
    -10.0,
    -1.23456,
    -1,
    -0.5,
    0.0,
    0.5,
    1.23456,
    10.0,
]  # so we don't overflow 32-bit precision
unit_range_test_values = [-1.0, -0.75, -0.5, -0.25, 0.0, 0.25, 0.5, 0.75, 1.0]

functions = [
    ("sqrt", sqrt, test_values),
    ("exp", exp, test_values_small),
    ("log", log, test_values),
    ("cos", cos, test_values),
    ("sin", sin, test_values),
    ("tan", tan, test_values),
    ("acos", acos, unit_range_test_values),
    ("asin", asin, unit_range_test_values),
    ("atan", atan, test_values),
    ("ceil", ceil, test_values),
    ("fabs", fabs, test_values),
    ("floor", floor, test_values),
    ("trunc", trunc, test_values),
    ("radians", radians, test_values),
    ("degrees", degrees, test_values),
]

for function_name, function, test_vals in functions:
    for value in test_vals:
        try:
            ans = "{:.5g}".format(function(value))
        except ValueError as e:
            ans = str(e)
        print("{}({:.5g}) = {}".format(function_name, value, ans))

tuple_functions = [
    ("frexp", frexp, test_values),
    ("modf", modf, test_values),
]

for function_name, function, test_vals in tuple_functions:
    for value in test_vals:
        x, y = function(value)
        print("{}({:.5g}) = ({:.5g}, {:.5g})".format(function_name, value, x, y))

binary_functions = [
    (
        "copysign",
        copysign,
        [(23.0, 42.0), (-23.0, 42.0), (23.0, -42.0), (-23.0, -42.0), (1.0, 0.0), (1.0, -0.0)],
    ),
    ("pow", pow, ((1.0, 0.0), (0.0, 1.0), (2.0, 0.5), (-3.0, 5.0), (-3.0, -4.0))),
    ("atan2", atan2, ((1.0, 0.0), (0.0, 1.0), (2.0, 0.5), (-3.0, 5.0), (-3.0, -4.0))),
    ("fmod", fmod, ((1.0, 1.0), (0.0, 1.0), (2.0, 0.5), (-3.0, 5.0), (-3.0, -4.0))),
    ("ldexp", ldexp, ((1.0, 0), (0.0, 1), (2.0, 2), (3.0, -2), (-3.0, -4))),
    (
        "log",
        log,
        (
            (2.0, 2.0),
            (3.0, 2.0),
            (4.0, 5.0),
            (0.0, 1.0),
            (1.0, 0.0),
            (-1.0, 1.0),
            (1.0, -1.0),
            (2.0, 1.0),
        ),
    ),
]

for function_name, function, test_vals in binary_functions:
    for value1, value2 in test_vals:
        try:
            ans = "{:.5g}".format(function(value1, value2))
        except (ValueError, ZeroDivisionError) as e:
            ans = type(e)
        print("{}({:.5g}, {:.5g}) = {}".format(function_name, value1, value2, ans))



# test integer floor division and modulo

# test all combination of +/-/0 cases
for i in range(-2, 3):
    for j in range(-4, 5):
        if j != 0:
            print(i, j, i // j, i % j, divmod(i, j))



import near
import json

@near.export
def hello_world():
  input_bytes = near.input(0)
  near.log_utf8("hello_world(): input(0): " + input_bytes.decode("ascii"))
  near.value_return(json.dumps({"input_length": len(input_bytes)}))



# Test MicroPython-specific features of collections.deque.

try:
    from collections import deque
except ImportError:
    print("SKIP")
    raise SystemExit


# Only fixed-size deques are supported, so length arg is mandatory.
try:
    deque(())
except TypeError:
    print("TypeError")

# Test third argument: flags when True means check for under/overflow
d = deque((), 2, True)

try:
    d.popleft()
except IndexError:
    print("IndexError")

try:
    d.pop()
except IndexError:
    print("IndexError")

# Removing elements with del is not supported, fallback to
# mp_obj_subscr() error message.
try:
    del d[0]
except TypeError:
    print("TypeError")

print(d.append(1))
print(d.popleft())

d.append(2)
print(d.popleft())

d.append(3)
d.append(4)
print(d.popleft(), d.popleft())
try:
    d.popleft()
except IndexError as e:
    print(repr(e))

try:
    d.pop()
except IndexError as e:
    print(repr(e))

d.append(5)
d.append(6)
print(len(d))
try:
    d.append(7)
except IndexError as e:
    print(repr(e))

try:
    d.appendleft(8)
except IndexError as e:
    print(repr(e))

print(len(d))

print(d.popleft(), d.popleft())
print(len(d))
try:
    d.popleft()
except IndexError as e:
    print(repr(e))



# MicroPython USB CDC REPL example
#
# Example demonstrating how to use os.dupterm() to provide the
# MicroPython REPL on a dynamic CDCInterface() serial port.
#
# To run this example:
#
# 1. Make sure `usb-device-cdc` is installed via: mpremote mip install usb-device-cdc
#
# 2. Run the example via: mpremote run cdc_repl_example.py
#
# 3. mpremote will exit with an error after the previous step, because when the
#    example runs the existing USB device disconnects and then re-enumerates with
#    the second serial port. If you check (for example by running mpremote connect
#    list) then you should now see two USB serial devices.
#
# 4. Connect to one of the new ports: mpremote connect PORTNAME
#
#    It may be necessary to type Ctrl-B to exit the raw REPL mode and resume the
#    interactive REPL after mpremote connects.
#
# MIT license; Copyright (c) 2023-2024 Angus Gratton
import os
import time
import usb.device
from usb.device.cdc import CDCInterface

cdc = CDCInterface()
cdc.init(timeout=0)  # zero timeout makes this non-blocking, suitable for os.dupterm()

# pass builtin_driver=True so that we get the built-in USB-CDC alongside,
# if it's available.
usb.device.get().init(cdc, builtin_driver=True)

print("Waiting for USB host to configure the interface...")

# wait for host enumerate as a CDC device...
while not cdc.is_open():
    time.sleep_ms(100)

# Note: This example doesn't wait for the host to access the new CDC port,
# which could be done by polling cdc.dtr, as this will block the REPL
# from resuming while this code is still executing.

print("CDC port enumerated, duplicating REPL...")

old_term = os.dupterm(cdc)



import pyb


def led_angle(seconds_to_run_for):
    # make LED objects
    l1 = pyb.LED(1)
    l2 = pyb.LED(2)
    accel = pyb.Accel()

    for i in range(20 * seconds_to_run_for):
        # get x-axis
        x = accel.x()

        # turn on LEDs depending on angle
        if x < -10:
            l1.on()
            l2.off()
        elif x > 10:
            l1.off()
            l2.on()
        else:
            l1.off()
            l2.off()

        # delay so that loop runs at at 1/50ms = 20Hz
        pyb.delay(50)



# Python 3.10+ functionality test for {} format string

def test(fmt, *args):
    print('{:8s}'.format(fmt) + '>' +  fmt.format(*args) + '<')

test("{:0s}", "ab")
test("{:06s}", "ab")
test("{:<06s}", "ab")
test("{:>06s}", "ab")



import sys

dir = "."
if "/" in __file__:
    dir = __file__.rsplit("/", 1)[0]

sys.path[0] = dir + "/os"
from path import *

assert split("") == ("", "")
assert split("path") == ("", "path")
assert split("/") == ("/", "")
assert split("/foo") == ("/", "foo")
assert split("/foo/") == ("/foo", "")
assert split("/foo/bar") == ("/foo", "bar")

assert exists(dir + "/test_path.py")
assert not exists(dir + "/test_path.py--")

assert isdir(dir + "/os")
assert not isdir(dir + "/os--")
assert not isdir(dir + "/test_path.py")

assert not isfile(dir + "/os")
assert isfile(dir + "/test_path.py")
assert not isfile(dir + "/test_path.py--")



# test that socket.connect() on a non-blocking socket raises EINPROGRESS
# and that an immediate write/send/read/recv does the right thing

import sys, errno, select, socket, ssl


def test(addr, hostname, block=True):
    print("---", hostname)
    s = socket.socket(socket.AF_INET)
    s.setblocking(block)
    try:
        s.connect(addr)
        print("connected")
    except OSError as e:
        if e.errno != errno.EINPROGRESS:
            raise
        print("EINPROGRESS")

    if sys.implementation.name != "micropython":
        # in CPython we have to wait, otherwise wrap_socket is not happy
        select.select([], [s], [])

    ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
    ssl_context.verify_mode = ssl.CERT_REQUIRED

    try:
        s = ssl_context.wrap_socket(s, do_handshake_on_connect=block, server_hostname=hostname)
        print("wrap: True")
    except OSError:
        print("wrap: error")

    if not block:
        try:
            while s.write(b"0") is None:
                pass
        except (ValueError, OSError):  # CPython raises ValueError, MicroPython raises OSError
            print("write: error")
    s.close()


if __name__ == "__main__":
    # connect to plain HTTP port, oops!
    addr = socket.getaddrinfo("micropython.org", 80, socket.AF_INET)[0][-1]
    test(addr, "micropython.org")
    # connect to plain HTTP port, oops!
    addr = socket.getaddrinfo("micropython.org", 80, socket.AF_INET)[0][-1]
    test(addr, "micropython.org", False)
    # connect to server with self-signed cert, oops!
    addr = socket.getaddrinfo("test.mosquitto.org", 8883, socket.AF_INET)[0][-1]
    test(addr, "test.mosquitto.org")



#!/usr/bin/env python3
#
# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2021 Philipp Ebensberger
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Evaluate FlexRAM configuration and generate startup code."""

import re
import argparse

# Regex for linker script configuration
ocram_regex = r"^\s*ocrm_size\s*=\s*(?P<size>.*);"
dtcm_regex = r"^\s*dtcm_size\s*=\s*(?P<size>.*);"
itcm_regex = r"^\s*itcm_size\s*=\s*(?P<size>.*);"

# Regex for GPR register base define in NXL hal
gpr_base_regex = r"^.*IOMUXC_GPR_BASE\s*\((?P<base_addr>\w*)u\)"

# Regex for FlexRAM parameters in NXP HAL
fsl_ram_bank_size_regex = r"^.*FSL_FEATURE_FLEXRAM_INTERNAL_RAM_BANK_SIZE\s*\((?P<size>\w*)\)"
fsl_bank_nbr_regex = (
    r"^.*FSL_FEATURE_FLEXRAM_INTERNAL_RAM_TOTAL_BANK_NUMBERS\s*\((?P<number>\w*)\)"
)


"""
According to AN12077:
    The minimum configuration of OCRAM is 64 KB. This is required
    due to ROM code requires at least 64 KB of RAM for its execution.

    2.1.1.1. Static configuration - Page 4
"""
ocram_min_size = 0x00010000  # 64 KB


# Value parser
def mimxrt_default_parser(defines_file, features_file, ld_script):
    with open(ld_script, "r") as input_file:
        input_str = input_file.read()
    #
    ocram_match = re.search(ocram_regex, input_str, re.MULTILINE)
    dtcm_match = re.search(dtcm_regex, input_str, re.MULTILINE)
    itcm_match = re.search(itcm_regex, input_str, re.MULTILINE)

    with open(defines_file, "r") as input_file:
        input_str = input_file.read()
    mcu_define_file_match = re.search(gpr_base_regex, input_str, re.MULTILINE)

    with open(features_file, "r") as input_file:
        input_str = input_file.read()
    fsl_ram_bank_size_match = re.search(fsl_ram_bank_size_regex, input_str, re.MULTILINE)
    fsl_bank_nbr_match = re.search(fsl_bank_nbr_regex, input_str, re.MULTILINE)
    #
    extract = {
        "ocram_size": int(ocram_match.group("size"), 16),
        "dtcm_size": int(dtcm_match.group("size"), 16),
        "itcm_size": int(itcm_match.group("size"), 16),
        "gpr_base_addr": int(mcu_define_file_match.group("base_addr"), 16),
        "fsl_ram_bank_size": int(fsl_ram_bank_size_match.group("size")),
        "fsl_bank_nbr": int(fsl_bank_nbr_match.group("number")),
    }
    # Evaluate configuration
    if extract["ocram_size"] < ocram_min_size:
        raise ValueError("OCRAM size must be at least {:08X}!".format(ocram_min_size))

    if (extract["ocram_size"] % extract["fsl_ram_bank_size"]) != 0:
        raise ValueError("Configuration invalid!")

    # Check if DTCM and ITCM size is either multiple of 32k or 4k,8k or 16k
    if extract["dtcm_size"] != 0x0:
        if extract["dtcm_size"] % extract["fsl_ram_bank_size"] != 0:
            if extract["dtcm_size"] not in (0x00000000, 0x00001000, 0x00002000, 0x00004000):
                raise ValueError("Configuration invalid!")

    if extract["itcm_size"] != 0x0:
        if extract["itcm_size"] % extract["fsl_ram_bank_size"] != 0:
            if extract["itcm_size"] not in (0x00000000, 0x00001000, 0x00002000, 0x00004000):
                raise ValueError("Configuration invalid!")
    #
    return extract


# Code generators
def mimxrt_default_gen_code(extract_dict):
    flexram_bank_cfg = "0b"
    avail_flexram = extract_dict["fsl_ram_bank_size"] * extract_dict["fsl_bank_nbr"]

    if (
        extract_dict["ocram_size"] + extract_dict["dtcm_size"] + extract_dict["itcm_size"]
    ) > avail_flexram:
        raise ValueError("Configuration exceeds available FlexRAM!")

    bit_patterns = (
        (extract_dict["ocram_size"], "01"),
        (extract_dict["dtcm_size"], "10"),
        (extract_dict["itcm_size"], "11"),
    )

    for size, pattern in bit_patterns:
        for _ in range(0, size, extract_dict["fsl_ram_bank_size"]):
            flexram_bank_cfg += pattern

    # Generate GPR Register config
    print(".equ __iomux_gpr14_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x38))
    print(".equ __iomux_gpr16_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x40))
    print(".equ __iomux_gpr17_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x44))
    print(
        ".equ __iomux_gpr17_value, 0x{:08X} /* {}k OCRAM, {}k DTCM, {}k ITCM */".format(
            int(flexram_bank_cfg, 2),
            extract_dict["ocram_size"] // 1024,
            extract_dict["dtcm_size"] // 1024,
            extract_dict["itcm_size"] // 1024,
        )
    )


def mimxrt_106x_gen_code(extract_dict):
    flexram_bank_cfg = "0b"
    avail_flexram = extract_dict["fsl_ram_bank_size"] * extract_dict["fsl_bank_nbr"]
    flexram_configurable_ocram = (
        extract_dict["ocram_size"] % 524288
    )  # 512kB OCRAM are not part of FlexRAM configurable memory

    if (
        flexram_configurable_ocram + extract_dict["dtcm_size"] + extract_dict["itcm_size"]
    ) > avail_flexram:
        raise ValueError("Configuration exceeds available FlexRAM!")

    for size, pattern in (
        (flexram_configurable_ocram, "01"),
        (extract_dict["dtcm_size"], "10"),
        (extract_dict["itcm_size"], "11"),
    ):
        for _ in range(0, size, extract_dict["fsl_ram_bank_size"]):
            flexram_bank_cfg += pattern

    # Generate GPR Register config
    print(".equ __iomux_gpr14_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x38))
    print(".equ __iomux_gpr16_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x40))
    print(".equ __iomux_gpr17_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x44))
    print(
        ".equ __iomux_gpr17_value, 0x{:08X} /* {}k OCRAM (512k OCRAM, {}k from FlexRAM), {}k DTCM, {}k ITCM */".format(
            int(flexram_bank_cfg, 2),
            extract_dict["ocram_size"] // 1024,
            flexram_configurable_ocram // 1024,
            extract_dict["dtcm_size"] // 1024,
            extract_dict["itcm_size"] // 1024,
        )
    )


def mimxrt_1176_gen_code(extract_dict):
    flexram_bank_cfg = "0b"
    avail_flexram = extract_dict["fsl_ram_bank_size"] * extract_dict["fsl_bank_nbr"]
    flexram_configurable_ocram = (
        extract_dict["ocram_size"] % 524288
    )  # 512kB OCRAM are not part of FlexRAM configurable memory

    if (
        flexram_configurable_ocram + extract_dict["dtcm_size"] + extract_dict["itcm_size"]
    ) > avail_flexram:
        raise ValueError("Configuration exceeds available FlexRAM!")

    for size, pattern in (
        (flexram_configurable_ocram, "01"),
        (extract_dict["dtcm_size"], "10"),
        (extract_dict["itcm_size"], "11"),
    ):
        for _ in range(0, size, extract_dict["fsl_ram_bank_size"]):
            flexram_bank_cfg += pattern

    # Generate GPR Register config
    print(".equ __iomux_gpr14_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x38))
    print(".equ __iomux_gpr16_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x40))
    print(".equ __iomux_gpr17_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x44))
    print(".equ __iomux_gpr18_adr, 0x{:08X}".format(extract_dict["gpr_base_addr"] + 0x48))
    print(
        ".equ __iomux_gpr17_value, 0x{:08X} /* {}k OCRAM (512k OCRAM, {}k from FlexRAM), {}k DTCM, {}k ITCM */".format(
            int(flexram_bank_cfg, 2) & 0xFFFF,
            extract_dict["ocram_size"] // 1024,
            flexram_configurable_ocram // 1024,
            extract_dict["dtcm_size"] // 1024,
            extract_dict["itcm_size"] // 1024,
        )
    )
    print(".equ __iomux_gpr18_value, 0x{:08X}".format((int(flexram_bank_cfg, 2) >> 16) & 0xFFFF))


def main(defines_file, features_file, ld_script, controller):
    dispatcher = {
        "MIMXRT1011": (mimxrt_default_parser, mimxrt_default_gen_code),
        "MIMXRT1015": (mimxrt_default_parser, mimxrt_default_gen_code),
        "MIMXRT1021": (mimxrt_default_parser, mimxrt_default_gen_code),
        "MIMXRT1052": (mimxrt_default_parser, mimxrt_default_gen_code),
        "MIMXRT1062": (mimxrt_default_parser, mimxrt_106x_gen_code),
        "MIMXRT1064": (mimxrt_default_parser, mimxrt_106x_gen_code),
        "MIMXRT1176": (mimxrt_default_parser, mimxrt_1176_gen_code),
    }

    extractor, code_generator = dispatcher[controller]

    extract_dict = extractor(defines_file, features_file, ld_script)
    code_generator(extract_dict)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="make-flexram-ld.py",
        usage="%(prog)s [options] [command]",
        description="Evaluate FlexRAM configuration and generate startup code.",
    )
    parser.add_argument(
        "-d",
        "--defines_file",
        dest="defines_file",
        help="Path to MCU defines file",
        default="../../../lib/nxp_driver/sdk/devices/MIMXRT1021/MIMXRT1021.h",
    )
    parser.add_argument(
        "-f",
        "--features_file",
        dest="features_file",
        help="Path to MCU features file",
        default="../../../lib/nxp_driver/sdk/devices/MIMXRT1021/MIMXRT1021_features.h",
    )
    parser.add_argument(
        "-l",
        "--ld_file",
        dest="linker_file",
        help="Path to the aggregated linker-script",
        default="MIMXRT1021.ld",
    )
    parser.add_argument(
        "-c", "--controller", dest="controller", help="Controller name", default="MIMXRT1021"
    )
    #
    args = parser.parse_args()
    main(args.defines_file, args.features_file, args.linker_file, args.controller)



# test _thread.exit() function
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

import time
import _thread


def thread_entry():
    _thread.exit()


for i in range(2):
    while True:
        try:
            _thread.start_new_thread(thread_entry, ())
            break
        except OSError:
            pass

# wait for threads to finish
time.sleep(1)
print("done")



import unittest


class TestWithRunTest(unittest.TestCase):
    run = False

    def runTest(self):
        TestWithRunTest.run = True

    def testRunTest(self):
        self.fail()

    @staticmethod
    def tearDownClass():
        if not TestWithRunTest.run:
            raise ValueError()


def test_func():
    pass


@unittest.expectedFailure
def test_foo():
    raise ValueError()


if __name__ == "__main__":
    unittest.main()



# Test current_task() function

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


async def task(result):
    result[0] = asyncio.current_task()


async def main():
    result = [None]
    t = asyncio.create_task(task(result))
    await asyncio.sleep(0)
    await asyncio.sleep(0)
    print(t is result[0])


try:
    print(asyncio.current_task())
except RuntimeError:
    print("RuntimeError")


asyncio.run(main())

try:
    print(asyncio.current_task())
except RuntimeError:
    print("RuntimeError")



# MIT license; Copyright (c) 2023 Jim Mussared
# Originally ported from CPython by Paul Sokolovsky

from ._sha256 import sha256


class sha224(sha256):
    digest_size = digestsize = 28
    _iv = [
        0xC1059ED8,
        0x367CD507,
        0x3070DD17,
        0xF70E5939,
        0xFFC00B31,
        0x68581511,
        0x64F98FA7,
        0xBEFA4FA4,
    ]



# test reading a given number of characters


def do(mode):
    if mode == "rb":
        enc = None
    else:
        enc = "utf-8"
    f = open("data/utf-8_2.txt", mode=mode, encoding=enc)
    print(f.read(1))
    print(f.read(1))
    print(f.read(2))
    print(f.read(4))

    # skip to end of line
    f.readline()

    # check 3-byte utf-8 char
    print(f.read(1 if mode == "rt" else 3))

    # check 4-byte utf-8 char
    print(f.read(1 if mode == "rt" else 4))

    f.close()


do("rb")
do("rt")



# Test simple HTTP request with asyncio.open_connection()

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


async def http_get(url):
    reader, writer = await asyncio.open_connection(url, 80)

    print("write GET")
    writer.write(b"GET / HTTP/1.0\r\n\r\n")
    await writer.drain()

    print("read response")
    data = await reader.read(100)
    print("read:", data.split(b"\r\n")[0])

    print("close")
    writer.close()
    await writer.wait_closed()
    print("done")


asyncio.run(http_get("micropython.org"))



from random import *


UPPER = 100

s = set()

# This number of course depends on a particular PRNG and its default seed
# as used by MicroPython.
for c in range(496):
    r = randrange(UPPER)
    s.add(r)

assert len(s) == UPPER



# test MicroPython-specific features of struct

try:
    import struct
except ImportError:
    print("SKIP")
    raise SystemExit

class A():
    pass

# pack and unpack objects
o = A()
s = struct.pack("<O", o)
o2 = struct.unpack("<O", s)
print(o is o2[0])

# pack can accept less arguments than required for the format spec
print(struct.pack('<2I', 1))

# pack and unpack pointer to a string
# This requires uctypes to get the address of the string and instead of
# putting this in a dedicated test that can be skipped we simply pass
# if the import fails.
try:
    import uctypes
    o = uctypes.addressof('abc')
    s = struct.pack("<S", o)
    o2 = struct.unpack("<S", s)
    assert o2[0] == 'abc'
except ImportError:
    pass



try:
    frozenset
except NameError:
    print("SKIP")
    raise SystemExit

sets = [
    frozenset(), frozenset({1}), frozenset({1, 2}), frozenset({1, 2, 3}), frozenset({2, 3}),
    frozenset({2, 3, 5}), frozenset({5}), frozenset({7})
]
for s in sets:
    for t in sets:
        print(sorted(s), '|', sorted(t), '=', sorted(s | t))
        print(sorted(s), '^', sorted(t), '=', sorted(s ^ t))
        print(sorted(s), '&', sorted(t), '=', sorted(s & t))
        print(sorted(s), '-', sorted(t), '=', sorted(s - t))
        u = s.copy()
        u |= t
        print(sorted(s), "|=", sorted(t), '-->', sorted(u))
        u = s.copy()
        u ^= t
        print(sorted(s), "^=", sorted(t), '-->', sorted(u))
        u = s.copy()
        u &= t
        print(sorted(s), "&=", sorted(t), "-->", sorted(u))
        u = s.copy()
        u -= t
        print(sorted(s), "-=", sorted(t), "-->", sorted(u))

        print(sorted(s), '==', sorted(t), '=', s == t)
        print(sorted(s), '!=', sorted(t), '=', s != t)
        print(sorted(s), '>', sorted(t), '=', s > t)
        print(sorted(s), '>=', sorted(t), '=', s >= t)
        print(sorted(s), '<', sorted(t), '=', s < t)
        print(sorted(s), '<=', sorted(t), '=', s <= t)



# Reads the USB VID and PID from the file specified by sys.argv[1] and then
# inserts those values into the template file specified by sys.argv[2],
# printing the result to stdout

from __future__ import print_function

import sys
import re
import string

config_prefix = "MICROPY_HW_USB_"
needed_keys = ("USB_PID_CDC_MSC", "USB_PID_CDC_HID", "USB_PID_CDC", "USB_VID")


def parse_usb_ids(filename):
    rv = dict()
    for line in open(filename).readlines():
        line = line.rstrip("\r\n")
        match = re.match(r"^#define\s+(\w+)\s+\(0x([0-9A-Fa-f]+)\)$", line)
        if match and match.group(1).startswith(config_prefix):
            key = match.group(1).replace(config_prefix, "USB_")
            val = match.group(2)
            # print("key =", key, "val =", val)
            if key in needed_keys:
                rv[key] = val
    for k in needed_keys:
        if k not in rv:
            raise Exception("Unable to parse %s from %s" % (k, filename))
    return rv


if __name__ == "__main__":
    usb_ids_file = sys.argv[1]
    template_file = sys.argv[2]
    replacements = parse_usb_ids(usb_ids_file)
    for line in open(template_file, "r").readlines():
        print(string.Template(line).safe_substitute(replacements), end="")



# Test general exception handling

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


# main task raising an exception
async def main():
    print("main start")
    raise ValueError(1)
    print("main done")


try:
    asyncio.run(main())
except ValueError as er:
    print("ValueError", er.args[0])


# sub-task raising an exception
async def task():
    print("task start")
    raise ValueError(2)
    print("task done")


async def main():
    print("main start")
    t = asyncio.create_task(task())
    await t
    print("main done")


try:
    asyncio.run(main())
except ValueError as er:
    print("ValueError", er.args[0])


# main task raising an exception with sub-task not yet scheduled
# TODO not currently working, task is never scheduled
async def task():
    # print('task run') uncomment this line when it works
    pass


async def main():
    print("main start")
    asyncio.create_task(task())
    raise ValueError(3)
    print("main done")


try:
    asyncio.run(main())
except ValueError as er:
    print("ValueError", er.args[0])



import sys

# ruff: noqa: E402
sys.path.insert(0, ".")
import aiohttp
import asyncio

try:
    URL = sys.argv[1]  # expects a websocket echo server
except Exception:
    URL = "ws://echo.websocket.events"


sslctx = False

if URL.startswith("wss:"):
    try:
        import ssl

        sslctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
        sslctx.verify_mode = ssl.CERT_NONE
    except Exception:
        pass


async def ws_test_echo(session):
    async with session.ws_connect(URL, ssl=sslctx) as ws:
        await ws.send_str("hello world!\r\n")
        async for msg in ws:
            if msg.type == aiohttp.WSMsgType.TEXT:
                print(msg.data)

            if "close" in msg.data:
                break
            await ws.send_str("close\r\n")
        await ws.close()


async def main():
    async with aiohttp.ClientSession() as session:
        await ws_test_echo(session)


if __name__ == "__main__":
    asyncio.run(main())



# test errors from bad operations (unary, binary, etc)

# unsupported unary operators
try:
    ~None
except TypeError:
    print('TypeError')
try:
    ~''
except TypeError:
    print('TypeError')
try:
    ~[]
except TypeError:
    print('TypeError')

# unsupported binary operators
try:
    False in True
except TypeError:
    print('TypeError')
try:
    1 * {}
except TypeError:
    print('TypeError')
try:
    1 in 1
except TypeError:
    print('TypeError')

# unsupported subscription
try:
    1[0] = 1
except TypeError:
    print('TypeError')
try:
    'a'[0] = 1
except TypeError:
    print('TypeError')
try:
    del 1[0]
except TypeError:
    print('TypeError')

# not an iterator
try:
    next(1)
except TypeError:
    print('TypeError')

# must be an exception type
try:
    raise 1
except TypeError:
    print('TypeError')

# no such name in import
try:
    from sys import youcannotimportmebecauseidontexist
except ImportError:
    print('ImportError')



# Helpers for generating BLE advertising payloads.

# A more fully-featured (and easier to use) version of this is implemented in
# aioble. This code is provided just as a basic example. See
# https://github.com/micropython/micropython-lib/tree/master/micropython/bluetooth/aioble

from micropython import const
import struct
import bluetooth

# Advertising payloads are repeated packets of the following form:
#   1 byte data length (N + 1)
#   1 byte type (see constants below)
#   N bytes type-specific data

_ADV_TYPE_FLAGS = const(0x01)
_ADV_TYPE_NAME = const(0x09)
_ADV_TYPE_UUID16_COMPLETE = const(0x3)
_ADV_TYPE_UUID32_COMPLETE = const(0x5)
_ADV_TYPE_UUID128_COMPLETE = const(0x7)
_ADV_TYPE_UUID16_MORE = const(0x2)
_ADV_TYPE_UUID32_MORE = const(0x4)
_ADV_TYPE_UUID128_MORE = const(0x6)
_ADV_TYPE_APPEARANCE = const(0x19)

_ADV_MAX_PAYLOAD = const(31)


# Generate a payload to be passed to gap_advertise(adv_data=...).
def advertising_payload(limited_disc=False, br_edr=False, name=None, services=None, appearance=0):
    payload = bytearray()

    def _append(adv_type, value):
        nonlocal payload
        payload += struct.pack("BB", len(value) + 1, adv_type) + value

    _append(
        _ADV_TYPE_FLAGS,
        struct.pack("B", (0x01 if limited_disc else 0x02) + (0x18 if br_edr else 0x04)),
    )

    if name:
        _append(_ADV_TYPE_NAME, name)

    if services:
        for uuid in services:
            b = bytes(uuid)
            if len(b) == 2:
                _append(_ADV_TYPE_UUID16_COMPLETE, b)
            elif len(b) == 4:
                _append(_ADV_TYPE_UUID32_COMPLETE, b)
            elif len(b) == 16:
                _append(_ADV_TYPE_UUID128_COMPLETE, b)

    # See org.bluetooth.characteristic.gap.appearance.xml
    if appearance:
        _append(_ADV_TYPE_APPEARANCE, struct.pack("<h", appearance))

    if len(payload) > _ADV_MAX_PAYLOAD:
        raise ValueError("advertising payload too large")

    return payload


def decode_field(payload, adv_type):
    i = 0
    result = []
    while i + 1 < len(payload):
        if payload[i + 1] == adv_type:
            result.append(payload[i + 2 : i + payload[i] + 1])
        i += 1 + payload[i]
    return result


def decode_name(payload):
    n = decode_field(payload, _ADV_TYPE_NAME)
    return str(n[0], "utf-8") if n else ""


def decode_services(payload):
    services = []
    for u in decode_field(payload, _ADV_TYPE_UUID16_COMPLETE):
        services.append(bluetooth.UUID(struct.unpack("<h", u)[0]))
    for u in decode_field(payload, _ADV_TYPE_UUID32_COMPLETE):
        services.append(bluetooth.UUID(struct.unpack("<d", u)[0]))
    for u in decode_field(payload, _ADV_TYPE_UUID128_COMPLETE):
        services.append(bluetooth.UUID(u))
    return services


def demo():
    payload = advertising_payload(
        name="micropython",
        services=[bluetooth.UUID(0x181A), bluetooth.UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")],
    )
    print(payload)
    print(decode_name(payload))
    print(decode_services(payload))


if __name__ == "__main__":
    demo()



# cmdline: -v -v
# Test constant-related bytecode optimisations
# (constant folding, compile-time if/while evaluation, etc.)
from micropython import const
import sys

try:
    sys.settrace
    # if MICROPY_PY_SYS_SETTRACE is enabled, compile-time const optimizations
    # are disabled so the bytecode output is very different
    print("SKIP")
    raise SystemExit
except AttributeError:
    pass

_STR = const("foo")
_EMPTY_TUPLE = const(())
_TRUE = const(True)
_FALSE = const(False)
_SMALLINT = const(33)
_ZERO = const(0)

# Bytecode generated for these if/while statements should contain no JUMP_IF
# and no instances of string 'Eliminated'

if _STR or _EMPTY_TUPLE:
    print("Kept")
if _STR and _EMPTY_TUPLE:
    print("Eliminated")
if _TRUE:
    print("Kept")
if _SMALLINT:
    print("Kept")
if _ZERO and _SMALLINT:
    print("Eliminated")
if _FALSE:
    print("Eliminated")

while _SMALLINT:
    print("Kept")
    break
while _ZERO:
    print("Eliminated")
while _FALSE:
    print("Eliminated")

# These values are stored in variables, and therefore bytecode will contain JUMP_IF

a = _EMPTY_TUPLE or _STR
if a == _STR:
    print("Kept")

b = _SMALLINT and _STR
if b == _STR:
    print("Kept")

# The compiler is also unable to optimise these expressions, even though the arguments are const,
# so these also contain JUMP_IF

if (_EMPTY_TUPLE or _STR) == _STR:
    print("Kept")

if (_EMPTY_TUPLE and _STR) == _STR:
    print("Not Eliminated")

if (not _STR) == _FALSE:
    print("Kept")

assert True



# This is an example on how to access accelerometer on
# PyBoard directly using I2C bus. As such, it's more
# intended to be an I2C example, rather than accelerometer
# example. For the latter, using pyb.Accel class is
# much easier.

from machine import Pin
from machine import I2C
import time

# Accelerometer needs to be powered on first. Even
# though signal is called "AVDD", and there's separate
# "DVDD", without AVDD, it won't event talk on I2C bus.
accel_pwr = Pin("MMA_AVDD")
accel_pwr.value(1)

i2c = I2C(1, baudrate=100000)
addrs = i2c.scan()
print("Scanning devices:", [hex(x) for x in addrs])
if 0x4C not in addrs:
    print("Accelerometer is not detected")

ACCEL_ADDR = 0x4C
ACCEL_AXIS_X_REG = 0
ACCEL_MODE_REG = 7

# Now activate measurements
i2c.mem_write(b"\x01", ACCEL_ADDR, ACCEL_MODE_REG)

print("Try to move accelerometer and watch the values")
while True:
    val = i2c.mem_read(1, ACCEL_ADDR, ACCEL_AXIS_X_REG)
    print(val[0])
    time.sleep(1)



# Test custom pollable objects implemented in Python.

from micropython import const

try:
    import socket, select, io
except ImportError:
    print("SKIP")
    raise SystemExit

_MP_STREAM_POLL = const(3)
_MP_STREAM_GET_FILENO = const(10)

_MP_STREAM_POLL_RD = const(0x0001)
_MP_STREAM_POLL_WR = const(0x0004)


def print_poll_output(lst):
    print([(type(obj), flags) for obj, flags in lst])


class CustomPollable(io.IOBase):
    def __init__(self):
        self.poll_state = 0

    def ioctl(self, cmd, arg):
        if cmd == _MP_STREAM_GET_FILENO:
            # Bare-metal ports don't call this ioctl, so don't print it.
            return -1

        print("CustomPollable.ioctl", cmd, arg)
        if cmd == _MP_STREAM_POLL:
            if self.poll_state == "delay_rd":
                self.poll_state = _MP_STREAM_POLL_RD
                return 0
            elif self.poll_state < 0:
                return self.poll_state
            else:
                return self.poll_state & arg


poller = select.poll()

# Use a new UDP socket for tests, which should be writable but not readable.
try:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind(socket.getaddrinfo("127.0.0.1", 8000)[0][-1])
except OSError:
    print("SKIP")
    raise SystemExit

x = CustomPollable()

# Register both a file-descriptor-based object and a custom pure-Python object.
poller.register(s)
poller.register(x)

# Modify the flags for the custom object.
poller.modify(x, select.POLLIN)

# Test polling.
print_poll_output(poller.poll(0))
x.poll_state = _MP_STREAM_POLL_WR
print_poll_output(poller.poll(0))
x.poll_state = _MP_STREAM_POLL_RD
print_poll_output(poller.poll(0))

# The custom object becomes readable only after being polled.
poller.modify(s, select.POLLIN)
x.poll_state = "delay_rd"
print_poll_output(poller.poll())

# The custom object returns an error.
x.poll_state = -1000
try:
    poller.poll(0)
except OSError as er:
    print("OSError", er.errno)

# Register then unregister a socket (a native stream), then test
# that the Python object is still pollable.
s2 = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
x.poll_state = _MP_STREAM_POLL_RD
poller.register(s2)
poller.unregister(s2)
print_poll_output(poller.poll())

# Test registering and unregistering multiple times.
for _ in range(2):
    poller.unregister(s)
    poller.unregister(x)
    poller.register(s2)
    poller.register(s, select.POLLIN)
    poller.register(x, select.POLLIN)
    poller.unregister(s2)
    print_poll_output(poller.poll())

# Clean up.
poller.unregister(x)
poller.unregister(s)
s2.close()
s.close()



# Reimplement, because CPython3.3 impl is rather bloated
import os
from collections import namedtuple

_ntuple_diskusage = namedtuple("usage", ("total", "used", "free"))


def rmtree(d):
    if not d:
        raise ValueError

    for name, type, *_ in os.ilistdir(d):
        path = d + "/" + name
        if type & 0x4000:  # dir
            rmtree(path)
        else:  # file
            os.unlink(path)
    os.rmdir(d)


def copyfileobj(src, dest, length=512):
    if hasattr(src, "readinto"):
        buf = bytearray(length)
        while True:
            sz = src.readinto(buf)
            if not sz:
                break
            if sz == length:
                dest.write(buf)
            else:
                b = memoryview(buf)[:sz]
                dest.write(b)
    else:
        while True:
            buf = src.read(length)
            if not buf:
                break
            dest.write(buf)


def disk_usage(path):
    bit_tuple = os.statvfs(path)
    blksize = bit_tuple[0]  # system block size
    total = bit_tuple[2] * blksize
    free = bit_tuple[3] * blksize
    used = total - free

    return _ntuple_diskusage(total, used, free)



# MicroPython package installer command line
# MIT license; Copyright (c) 2022 Jim Mussared

import argparse
import sys


def do_install():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-t",
        "--target",
        help="Directory to start discovery",
    )
    parser.add_argument(
        "-i",
        "--index",
        help="Pattern to match test files",
    )
    parser.add_argument(
        "--mpy",
        action="store_true",
        help="download as compiled .mpy files (default)",
    )
    parser.add_argument(
        "--no-mpy",
        action="store_true",
        help="download as .py source files",
    )
    parser.add_argument("package", nargs="+")
    args = parser.parse_args(args=sys.argv[2:])

    from . import install

    for package in args.package:
        version = None
        if "@" in package:
            package, version = package.split("@")
        install(package, args.index, args.target, version, not args.no_mpy)


if len(sys.argv) >= 2:
    if sys.argv[1] == "install":
        do_install()
    else:
        print('mip: Unknown command "{}"'.format(sys.argv[1]))



# test construction of bytearray from different objects
try:
    from array import array
except ImportError:
    print("SKIP")
    raise SystemExit

# arrays
print(bytearray(array('h', [1, 2])))
print(bytearray(array('I', [1, 2])))



# Test blit between different color spaces
try:
    import framebuf, sys
except ImportError:
    print("SKIP")
    raise SystemExit

# Monochrome glyph/icon
w = 8
h = 8
cbuf = bytearray(w * h // 8)
fbc = framebuf.FrameBuffer(cbuf, w, h, framebuf.MONO_HLSB)
fbc.line(0, 0, 7, 7, 1)

# RGB565 destination
wd = 16
hd = 16
dest = bytearray(wd * hd * 2)
fbd = framebuf.FrameBuffer(dest, wd, hd, framebuf.RGB565)

wp = 2
bg = 0x1234
fg = 0xF800
pal = bytearray(wp * 2)
palette = framebuf.FrameBuffer(pal, wp, 1, framebuf.RGB565)
palette.pixel(0, 0, bg)
palette.pixel(1, 0, fg)

fbd.blit(fbc, 0, 0, -1, palette)

print(fbd.pixel(0, 0) == fg)
print(fbd.pixel(7, 7) == fg)
print(fbd.pixel(8, 8) == 0)  # Outside blit
print(fbd.pixel(0, 1) == bg)
print(fbd.pixel(1, 0) == bg)



# Copyright (C) 2001-2006 Python Software Foundation
# Author: Barry Warsaw
# Contact: email-sig@python.org

"""email package exception classes."""


class MessageError(Exception):
    """Base class for errors in the email package."""


class MessageParseError(MessageError):
    """Base class for message parsing errors."""


class HeaderParseError(MessageParseError):
    """Error while parsing headers."""


class BoundaryError(MessageParseError):
    """Couldn't find terminating boundary."""


class MultipartConversionError(MessageError):  # , TypeError):
    """Conversion to a multipart is prohibited."""


class CharsetError(MessageError):
    """An illegal charset was given."""


# These are parsing defects which the parser was able to work around.
class MessageDefect(ValueError):
    """Base class for a message defect."""

    def __init__(self, line=None):
        if line is not None:
            super().__init__(line)
        self.line = line


class NoBoundaryInMultipartDefect(MessageDefect):
    """A message claimed to be a multipart but had no boundary parameter."""


class StartBoundaryNotFoundDefect(MessageDefect):
    """The claimed start boundary was never found."""


class CloseBoundaryNotFoundDefect(MessageDefect):
    """A start boundary was found, but not the corresponding close boundary."""


class FirstHeaderLineIsContinuationDefect(MessageDefect):
    """A message had a continuation line as its first header line."""


class MisplacedEnvelopeHeaderDefect(MessageDefect):
    """A 'Unix-from' header was found in the middle of a header block."""


class MissingHeaderBodySeparatorDefect(MessageDefect):
    """Found line with no leading whitespace and no colon before blank line."""


# XXX: backward compatibility, just in case (it was never emitted).
MalformedHeaderDefect = MissingHeaderBodySeparatorDefect


class MultipartInvariantViolationDefect(MessageDefect):
    """A message claimed to be a multipart but no subparts were found."""


class InvalidMultipartContentTransferEncodingDefect(MessageDefect):
    """An invalid content transfer encoding was set on the multipart itself."""


class UndecodableBytesDefect(MessageDefect):
    """Header contained bytes that could not be decoded"""


class InvalidBase64PaddingDefect(MessageDefect):
    """base64 encoded sequence had an incorrect length"""


class InvalidBase64CharactersDefect(MessageDefect):
    """base64 encoded sequence had characters not in base64 alphabet"""


# These errors are specific to header parsing.


class HeaderDefect(MessageDefect):
    """Base class for a header defect."""

    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)


class InvalidHeaderDefect(HeaderDefect):
    """Header is not valid, message gives details."""


class HeaderMissingRequiredValue(HeaderDefect):
    """A header that must have a value had none"""


class NonPrintableDefect(HeaderDefect):
    """ASCII characters outside the ascii-printable range found"""

    def __init__(self, non_printables):
        super().__init__(non_printables)
        self.non_printables = non_printables

    def __str__(self):
        return "the following ASCII non-printables found in header: " "{}".format(
            self.non_printables
        )


class ObsoleteHeaderDefect(HeaderDefect):
    """Header uses syntax declared obsolete by RFC 5322"""


class NonASCIILocalPartDefect(HeaderDefect):
    """local_part contains non-ASCII characters"""

    # This defect only occurs during unicode parsing, not when
    # parsing messages decoded from binary.



#!/usr/bin/env python3
#
# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2020 Jim Mussared
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# This script obfuscates the ST wireless binaries so they can be safely copied
# to the flash filesystem and not be accidentally discovered by the FUS during
# an update. See more information (and the corresponding de-obfuscation) in
# rfcore_firmware.py as well as instructions on how to use.

import os
import re
import struct
import sys
from binascii import crc32
from rfcore_firmware import validate_crc, _OBFUSCATION_KEY


_FIRMWARE_FILES = {
    "stm32wb5x_FUS_fw_1_0_2.bin": "fus_102.bin",
    "stm32wb5x_FUS_fw.bin": "fus_110.bin",
    "stm32wb5x_BLE_HCILayer_fw.bin": "ws_ble_hci.bin",
}
_RELEASE_NOTES = "Release_Notes.html"


def get_details(release_notes, filename):
    if not release_notes:
        return None
    file_details = re.findall(
        rb"%s,(((0x[\d\S]+?),)+[vV][\d\.]+)[<,]" % filename.encode(),
        release_notes,
        flags=re.DOTALL,
    )
    # The release note has all past version details also, but current is at top
    latest_details = file_details[0][0].split(b",")
    addr_1m, addr_640k, addr_512k, addr_256k, version = latest_details
    addr_1m = int(addr_1m, 0)
    addr_640k = int(addr_640k, 0)
    addr_512k = int(addr_512k, 0)
    addr_256k = int(addr_256k, 0)
    version = [int(v) for v in version.lower().lstrip(b"v").split(b".")]
    return addr_1m, addr_640k, addr_512k, addr_256k, version


def main(src_path, dest_path):
    # Load the release note to parse for important details
    with open(os.path.join(src_path, _RELEASE_NOTES), "rb") as f:
        release_notes = f.read()
    # remove some formatting
    release_notes = re.sub(rb"</?strong>", b"", release_notes)
    # flatten tables
    release_notes = re.sub(
        rb"</t[dh]>\W*\n*\W*",
        b",",
        release_notes.replace(b"<td>", b"").replace(b"<th>", b""),
    )
    if (
        b"Wireless Coprocessor Binary,STM32WB5xxG(1M),STM32WB5xxY(640k),STM32WB5xxE(512K),STM32WB5xxC(256K),Version,"
        not in release_notes
    ):
        raise SystemExit(
            "Cannot determine binary load address, please confirm Coprocessor folder / Release Notes format."
        )

    for src_filename, dest_file in _FIRMWARE_FILES.items():
        src_file = os.path.join(src_path, src_filename)
        dest_file = os.path.join(dest_path, dest_file)
        if not os.path.exists(src_file):
            print("Unable to find: {}".format(src_file))
            continue
        sz = 0
        with open(src_file, "rb") as src:
            crc = 0
            with open(dest_file, "wb") as dest:
                while True:
                    b = src.read(4)
                    if not b:
                        break
                    (v,) = struct.unpack("<I", b)
                    v ^= _OBFUSCATION_KEY
                    vs = struct.pack("<I", v)
                    dest.write(vs)
                    crc = crc32(vs, crc)
                    sz += 4

                addr_1m, addr_640k, addr_512k, addr_256k, version = get_details(
                    release_notes, src_filename
                )

                footer = struct.pack(
                    "<37sIIIIbbbI",
                    src_filename.encode(),
                    addr_1m,
                    addr_640k,
                    addr_512k,
                    addr_256k,
                    *version,
                    _OBFUSCATION_KEY,
                )
                assert len(footer) == 60
                dest.write(footer)
                crc = crc32(footer, crc)
                crc = 0xFFFFFFFF & -crc - 1
                dest.write(struct.pack("<I", crc))
                sz += 64

        print(
            f"Written {src_filename} v{version[0]}.{version[1]}.{version[2]} to {dest_file} ({sz} bytes)"
        )
        check_file_details(dest_file)


def check_file_details(filename):
    """Should match copy of function in rfcore_firmware.py to confirm operation"""
    with open(filename, "rb") as f:
        if not validate_crc(f):
            raise ValueError("file validation failed: incorrect crc")

        f.seek(-64, 2)
        footer = f.read()
        assert len(footer) == 64
        details = struct.unpack("<37sIIIIbbbII", footer)
        (
            src_filename,
            addr_1m,
            addr_640k,
            addr_512k,
            addr_256k,
            vers_major,
            vers_minor,
            vers_patch,
            KEY,
            crc,
        ) = details
        if KEY != _OBFUSCATION_KEY:
            raise ValueError("file validation failed: incorrect key")

    return (
        src_filename,
        addr_1m,
        addr_640k,
        addr_512k,
        addr_256k,
        (vers_major, vers_minor, vers_patch),
    )


if __name__ == "__main__":
    if len(sys.argv) != 3:
        print("Usage: {} src_path dest_path".format(sys.argv[0]))
        print()
        print(
            '"src_path" should be the location of the ST binaries from https://github.com/STMicroelectronics/STM32CubeWB/tree/master/Projects/STM32WB_Copro_Wireless_Binaries/STM32WB5x'
        )
        print(
            '"dest_path" will be where fus_102.bin, fus_110.bin, and ws_ble_hci.bin will be written to.'
        )
        sys.exit(1)

    main(sys.argv[1], sys.argv[2])



"""
Timer test for the CC3200 based boards.
"""

from machine import Timer
import os
import time

mch = os.uname().machine
if "LaunchPad" in mch:
    pwm_pin = "GP24"
elif "WiPy" in mch:
    pwm_pin = "GP24"
else:
    raise Exception("Board not supported!")

for i in range(4):
    tim = Timer(i, mode=Timer.PERIODIC)
    print(tim)
    ch = tim.channel(Timer.A, freq=5)
    print(ch)
    ch = tim.channel(Timer.B, freq=5)
    print(ch)
    tim = Timer(i, mode=Timer.ONE_SHOT)
    print(tim)
    ch = tim.channel(Timer.A, freq=50)
    print(ch)
    ch = tim.channel(Timer.B, freq=50)
    print(ch)
    tim = Timer(i, mode=Timer.PWM)
    print(tim)
    ch = tim.channel(Timer.A, freq=50000, duty_cycle=2000, polarity=Timer.POSITIVE)
    print(ch)
    ch = tim.channel(Timer.B, freq=50000, duty_cycle=8000, polarity=Timer.NEGATIVE)
    print(ch)
    tim.deinit()
    print(tim)

for i in range(4):
    tim = Timer(i, mode=Timer.PERIODIC)
    tim.deinit()


class TimerTest:
    def __init__(self):
        self.tim = Timer(0, mode=Timer.PERIODIC)
        self.int_count = 0

    def timer_isr(self, tim_ch):
        self.int_count += 1


timer_test = TimerTest()
ch = timer_test.tim.channel(Timer.A, freq=5)
print(ch.freq() == 5)
ch.irq(handler=timer_test.timer_isr, trigger=Timer.TIMEOUT)
time.sleep_ms(1001)
print(timer_test.int_count == 5)

ch.freq(100)
timer_test.int_count = 0
time.sleep_ms(1001)
print(timer_test.int_count == 100)

ch.freq(1000)
time.sleep_ms(1500)
timer_test.int_count = 0
time.sleep_ms(2000)
print(timer_test.int_count == 2000)

timer_test.tim.deinit()
timer_test.tim.init(mode=Timer.ONE_SHOT)
ch = timer_test.tim.channel(Timer.A, period=100000)
ch.irq(handler=timer_test.timer_isr, trigger=Timer.TIMEOUT)
timer_test.int_count = 0
time.sleep_ms(101)
print(timer_test.int_count == 1)
time.sleep_ms(101)
print(timer_test.int_count == 1)
timer_test.tim.deinit()
print(timer_test.tim)

# 32 bit modes
tim = Timer(0, mode=Timer.PERIODIC, width=32)
ch = tim.channel(Timer.A | Timer.B, period=5000000)

# check for memory leaks...
for i in range(1000):
    tim = Timer(0, mode=Timer.PERIODIC)
    ch = tim.channel(Timer.A, freq=5)

# next ones must fail
try:
    tim = Timer(0, mode=12)
except:
    print("Exception")

try:
    tim = Timer(4, mode=Timer.ONE_SHOT)
except:
    print("Exception")

try:
    tim = Timer(0, mode=Timer.PWM, width=32)
except:
    print("Exception")

tim = Timer(0, mode=Timer.PWM)

try:
    ch = tim.channel(TIMER_A | TIMER_B, freq=10)
except:
    print("Exception")

try:
    ch = tim.channel(TIMER_A, freq=4)
except:
    print("Exception")



from pyb import SPI

# test we can correctly create by id
for bus in (-1, 0, 1, 2):
    try:
        SPI(bus)
        print("SPI", bus)
    except ValueError:
        print("ValueError", bus)

spi = SPI(1)
print(spi)

spi = SPI(1, SPI.CONTROLLER)
spi = SPI(1, SPI.CONTROLLER, baudrate=500000)
spi = SPI(
    1, SPI.CONTROLLER, 500000, polarity=1, phase=0, bits=8, firstbit=SPI.MSB, ti=False, crc=None
)
print(str(spi)[:32], str(spi)[53:])  # don't print baudrate/prescaler

spi.init(SPI.PERIPHERAL, phase=1)
print(spi)
try:
    # need to flush input before we get an error (error is what we want to test)
    for i in range(10):
        spi.recv(1, timeout=100)
except OSError:
    print("OSError")

spi.init(SPI.CONTROLLER)
spi.send(1, timeout=100)
print(spi.recv(1, timeout=100))
print(spi.send_recv(1, timeout=100))

spi.deinit()



# Check that super rejects invalid arguments.
try:
    super(str, 0)
except TypeError:
    print("TypeError")

try:
    super(str, int)
except TypeError:
    print("TypeError")

try:
    super(0, int)
except TypeError:
    print("TypeError")



#!/usr/bin/env python3

# This file is part of the MicroPython project, http://micropython.org/
# The MIT License (MIT)
# Copyright (c) 2020 Damien P. George
#
# run-multitests.py
# Runs a test suite that relies on two micropython instances/devices
# interacting in some way. Typically used to test networking / bluetooth etc.


import sys, os, time, re, select
import argparse
import itertools
import subprocess
import tempfile

test_dir = os.path.abspath(os.path.dirname(__file__))

if os.path.abspath(sys.path[0]) == test_dir:
    # remove the micropython/tests dir from path to avoid
    # accidentally importing tests like micropython/const.py
    sys.path.pop(0)

sys.path.insert(0, test_dir + "/../tools")
import pyboard

if os.name == "nt":
    CPYTHON3 = os.getenv("MICROPY_CPYTHON3", "python3.exe")
    MICROPYTHON = os.path.abspath(
        os.getenv(
            "MICROPY_MICROPYTHON", test_dir + "/../ports/windows/build-standard/micropython.exe"
        )
    )
else:
    CPYTHON3 = os.getenv("MICROPY_CPYTHON3", "python3")
    MICROPYTHON = os.path.abspath(
        os.getenv("MICROPY_MICROPYTHON", test_dir + "/../ports/unix/build-standard/micropython")
    )

# For diff'ing test output
DIFF = os.getenv("MICROPY_DIFF", "diff -u")

PYTHON_TRUTH = CPYTHON3

INSTANCE_READ_TIMEOUT_S = 10

APPEND_CODE_TEMPLATE = """
import sys
class multitest:
    @staticmethod
    def flush():
        try:
            sys.stdout.flush()
        except AttributeError:
            pass
    @staticmethod
    def skip():
        print("SKIP")
        multitest.flush()
        raise SystemExit
    @staticmethod
    def next():
        print("NEXT")
        multitest.flush()
    @staticmethod
    def broadcast(msg):
        print("BROADCAST", msg)
        multitest.flush()
    @staticmethod
    def wait(msg):
        msg = "BROADCAST " + msg
        while True:
            if sys.stdin.readline().rstrip() == msg:
                return
    @staticmethod
    def globals(**gs):
        for g in gs:
            print("SET {{}} = {{!r}}".format(g, gs[g]))
        multitest.flush()
    @staticmethod
    def get_network_ip():
        try:
            ip = nic.ifconfig()[0]
        except:
            try:
                import network
                if hasattr(network, "WLAN"):
                    ip = network.WLAN().ifconfig()[0]
                else:
                    ip = network.LAN().ifconfig()[0]
            except:
                ip = HOST_IP
        return ip
    @staticmethod
    def expect_reboot(resume, delay_ms=0):
        print("WAIT_FOR_REBOOT", resume, delay_ms)
    @staticmethod
    def output_metric(data):
        print("OUTPUT_METRIC", data)

{}

instance{}()
multitest.flush()
"""

# The btstack implementation on Unix generates some spurious output that we
# can't control.  Also other platforms may output certain warnings/errors that
# can be safely ignored.
IGNORE_OUTPUT_MATCHES = (
    "libusb: error ",  # It tries to open devices that it doesn't have access to (libusb prints unconditionally).
    "hci_transport_h2_libusb.c",  # Same issue. We enable LOG_ERROR in btstack.
    "USB Path: ",  # Hardcoded in btstack's libusb transport.
    "hci_number_completed_packet",  # Warning from btstack.
    "lld_pdu_get_tx_flush_nb HCI packet count mismatch (",  # From ESP-IDF, see https://github.com/espressif/esp-idf/issues/5105
)


def get_host_ip(_ip_cache=[]):
    if not _ip_cache:
        try:
            import socket

            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(("8.8.8.8", 80))
            _ip_cache.append(s.getsockname()[0])
            s.close()
        except:
            _ip_cache.append("127.0.0.1")
    return _ip_cache[0]


class PyInstance:
    def __init__(self):
        pass

    def close(self):
        pass

    def prepare_script_from_file(self, filename, prepend, append):
        with open(filename, "rb") as f:
            script = f.read()
        if prepend:
            script = bytes(prepend, "ascii") + b"\n" + script
        if append:
            script += b"\n" + bytes(append, "ascii")
        return script

    def run_file(self, filename, prepend="", append=""):
        return self.run_script(self.prepare_script_from_file(filename, prepend, append))

    def start_file(self, filename, prepend="", append=""):
        return self.start_script(self.prepare_script_from_file(filename, prepend, append))


class PyInstanceSubProcess(PyInstance):
    def __init__(self, argv, env=None):
        self.argv = argv
        self.cwd = None
        self.env = {n: v for n, v in (i.split("=") for i in env)} if env else None
        self.popen = None
        self.finished = True

    def __str__(self):
        return self.argv[0].rsplit("/")[-1]

    def prepare_script_from_file(self, filename, prepend, append):
        # Make tests run in the directory of the test file, and in an isolated environment
        # (i.e. `import io` would otherwise get the `tests/io` directory).
        self.cwd = os.path.dirname(filename)
        remove_cwd_from_sys_path = b"import sys\nsys.path.remove('')\n\n"
        return remove_cwd_from_sys_path + super().prepare_script_from_file(
            filename, prepend, append
        )

    def run_script(self, script):
        output = b""
        err = None
        try:
            p = subprocess.run(
                self.argv,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                input=script,
                cwd=self.cwd,
                env=self.env,
            )
            output = p.stdout
        except subprocess.CalledProcessError as er:
            err = er
        return str(output.strip(), "ascii"), err

    def start_script(self, script):
        self.popen = subprocess.Popen(
            self.argv + ["-c", script],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=self.cwd,
            env=self.env,
        )
        self.finished = False

    def stop(self):
        if self.popen and self.popen.poll() is None:
            self.popen.terminate()

    def readline(self):
        sel = select.select([self.popen.stdout.raw], [], [], 0.001)
        if not sel[0]:
            self.finished = self.popen.poll() is not None
            return None, None
        out = self.popen.stdout.raw.readline()
        if out == b"":
            self.finished = self.popen.poll() is not None
            return None, None
        else:
            return str(out.rstrip(), "ascii"), None

    def write(self, data):
        self.popen.stdin.write(data)
        self.popen.stdin.flush()

    def is_finished(self):
        return self.finished

    def wait_finished(self):
        self.popen.wait()
        out = self.popen.stdout.read()
        return str(out, "ascii"), ""


class PyInstancePyboard(PyInstance):
    @staticmethod
    def map_device_shortcut(device):
        if device[0] == "a" and device[1:].isdigit():
            return "/dev/ttyACM" + device[1:]
        elif device[0] == "u" and device[1:].isdigit():
            return "/dev/ttyUSB" + device[1:]
        else:
            return device

    def __init__(self, device):
        device = self.map_device_shortcut(device)
        self.device = device
        self.pyb = pyboard.Pyboard(device)
        self.pyb.enter_raw_repl()
        self.finished = True

    def __str__(self):
        return self.device.rsplit("/")[-1]

    def close(self):
        self.pyb.exit_raw_repl()
        self.pyb.close()

    def run_script(self, script):
        output = b""
        err = None
        try:
            self.pyb.enter_raw_repl()
            output = self.pyb.exec_(script)
        except pyboard.PyboardError as er:
            err = er
        return str(output.strip(), "ascii"), err

    def start_script(self, script):
        self.pyb.enter_raw_repl()
        self.pyb.exec_raw_no_follow(script)
        self.finished = False

    def stop(self):
        self.pyb.serial.write(b"\r\x03")

    def readline(self):
        if self.finished:
            return None, None
        if self.pyb.serial.inWaiting() == 0:
            return None, None
        out = self.pyb.read_until(1, (b"\r\n", b"\x04"))
        if out.endswith(b"\x04"):
            self.finished = True
            out = out[:-1]
            err = str(self.pyb.read_until(1, b"\x04"), "ascii")
            err = err[:-1]
            if not out and not err:
                return None, None
        else:
            err = None
        return str(out.rstrip(), "ascii"), err

    def write(self, data):
        self.pyb.serial.write(data)

    def is_finished(self):
        return self.finished

    def wait_finished(self):
        out, err = self.pyb.follow(10, None)
        return str(out, "ascii"), str(err, "ascii")


def prepare_test_file_list(test_files):
    test_files2 = []
    for test_file in sorted(test_files):
        num_instances = 0
        with open(test_file) as f:
            for line in f:
                m = re.match(r"def instance([0-9]+)\(\):", line)
                if m:
                    num_instances = max(num_instances, int(m.group(1)) + 1)
        test_files2.append((test_file, num_instances))
    return test_files2


def trace_instance_output(instance_idx, line):
    if cmd_args.trace_output:
        t_ms = round((time.time() - trace_t0) * 1000)
        print("{:6} i{} :".format(t_ms, instance_idx), line)
        sys.stdout.flush()


def run_test_on_instances(test_file, num_instances, instances):
    global trace_t0
    trace_t0 = time.time()

    error = False
    skip = False
    injected_globals = ""
    output = [[] for _ in range(num_instances)]
    output_metrics = []

    # If the test calls get_network_ip() then inject HOST_IP so that devices can know
    # the IP address of the host.  Do this lazily to not require a TCP/IP connection
    # on the host if it's not needed.
    with open(test_file, "rb") as f:
        if b"get_network_ip" in f.read():
            injected_globals += "HOST_IP = '" + get_host_ip() + "'\n"

    if cmd_args.trace_output:
        print("TRACE {}:".format("|".join(str(i) for i in instances)))

    # Start all instances running, in order, waiting until they signal they are ready
    for idx in range(num_instances):
        append_code = APPEND_CODE_TEMPLATE.format(injected_globals, idx)
        instance = instances[idx]
        instance.start_file(test_file, append=append_code)
        last_read_time = time.time()
        while True:
            if instance.is_finished():
                break
            out, err = instance.readline()
            if out is None and err is None:
                if time.time() > last_read_time + INSTANCE_READ_TIMEOUT_S:
                    output[idx].append("TIMEOUT")
                    error = True
                    break
                time.sleep(0.1)
                continue
            last_read_time = time.time()
            if out is not None and not any(m in out for m in IGNORE_OUTPUT_MATCHES):
                trace_instance_output(idx, out)
                if out.startswith("SET "):
                    injected_globals += out[4:] + "\n"
                elif out == "SKIP":
                    skip = True
                    break
                elif out == "NEXT":
                    break
                else:
                    output[idx].append(out)
            if err is not None:
                trace_instance_output(idx, err)
                output[idx].append(err)
                error = True

        if error or skip:
            break

    if not error and not skip:
        # Capture output and wait for all instances to finish running
        last_read_time = [time.time() for _ in range(num_instances)]
        while True:
            num_running = 0
            num_output = 0
            for idx in range(num_instances):
                instance = instances[idx]
                if instance.is_finished():
                    continue
                num_running += 1
                out, err = instance.readline()
                if out is None and err is None:
                    if time.time() > last_read_time[idx] + INSTANCE_READ_TIMEOUT_S:
                        output[idx].append("TIMEOUT")
                        error = True
                    continue
                num_output += 1
                last_read_time[idx] = time.time()
                if out is not None and not any(m in out for m in IGNORE_OUTPUT_MATCHES):
                    trace_instance_output(idx, out)
                    if out.startswith("WAIT_FOR_REBOOT"):
                        _, resume, delay_ms = out.split(" ")

                        if wait_for_reboot(instance, delay_ms):
                            # Restart the test code, resuming from requested instance block
                            if not resume.startswith("instance{}".format(idx)):
                                raise SystemExit(
                                    'ERROR: resume function must start with "instance{}"'.format(
                                        idx
                                    )
                                )
                            append_code = APPEND_CODE_TEMPLATE.format(injected_globals, resume[8:])
                            instance.start_file(test_file, append=append_code)
                            last_read_time[idx] = time.time()

                    if out.startswith("BROADCAST "):
                        for instance2 in instances:
                            if instance2 is not instance:
                                instance2.write(bytes(out, "ascii") + b"\r\n")
                    elif out.startswith("OUTPUT_METRIC "):
                        output_metrics.append(out.split(" ", 1)[1])
                    else:
                        output[idx].append(out)
                if err is not None:
                    trace_instance_output(idx, err)
                    output[idx].append(err)
                    error = True

            if not num_output:
                time.sleep(0.1)
            if not num_running or error:
                break

    # Stop all instances
    for idx in range(num_instances):
        instances[idx].stop()

    output_str = ""
    for idx, lines in enumerate(output):
        output_str += "--- instance{} ---\n".format(idx)
        output_str += "\n".join(lines) + "\n"

    return error, skip, output_str, output_metrics


def wait_for_reboot(instance, extra_timeout_ms=0):
    # Monitor device responses for reboot banner, waiting for idle.
    extra_timeout = float(extra_timeout_ms) * 1000
    INITIAL_TIMEOUT = 1 + extra_timeout
    FULL_TIMEOUT = 5 + extra_timeout
    t_start = t_last_activity = time.monotonic()
    while True:
        t = time.monotonic()
        out, err = instance.readline()
        if err is not None:
            print("Reboot: communication error", err)
            return False
        if out:
            t_last_activity = t
            # Check for reboot banner, see py/pyexec.c "reset friendly REPL"
            if re.match(r"^MicroPython v\d+\.\d+\.\d+.* on .*; .* with .*$", out):
                time.sleep(0.1)
                break

        if t_last_activity == t_start:
            if t - t_start > INITIAL_TIMEOUT:
                print("Reboot: missed initial Timeout")
                return False
        else:
            if t - t_start > FULL_TIMEOUT:
                print("Reboot: Timeout")
                return False

    instance.pyb.enter_raw_repl()
    return True


def print_diff(a, b):
    a_fd, a_path = tempfile.mkstemp(text=True)
    b_fd, b_path = tempfile.mkstemp(text=True)
    os.write(a_fd, a.encode())
    os.write(b_fd, b.encode())
    os.close(a_fd)
    os.close(b_fd)
    subprocess.run(DIFF.split(" ") + [a_path, b_path])
    os.unlink(a_path)
    os.unlink(b_path)


def run_tests(test_files, instances_truth, instances_test):
    skipped_tests = []
    passed_tests = []
    failed_tests = []

    for test_file, num_instances in test_files:
        instances_str = "|".join(str(instances_test[i]) for i in range(num_instances))
        print("{} on {}: ".format(test_file, instances_str), end="")
        if cmd_args.show_output or cmd_args.trace_output:
            print()
        sys.stdout.flush()

        # Run test on test instances
        error, skip, output_test, output_metrics = run_test_on_instances(
            test_file, num_instances, instances_test
        )

        if not skip:
            # Check if truth exists in a file, and read it in
            test_file_expected = test_file + ".exp"
            if os.path.isfile(test_file_expected):
                with open(test_file_expected) as f:
                    output_truth = f.read()
            else:
                # Run test on truth instances to get expected output
                _, _, output_truth, _ = run_test_on_instances(
                    test_file, num_instances, instances_truth
                )

        if cmd_args.show_output:
            print("### TEST ###")
            print(output_test, end="")
            if not skip:
                print("### TRUTH ###")
                print(output_truth, end="")

        # Print result of test
        if skip:
            print("skip")
            skipped_tests.append(test_file)
        elif output_test == output_truth:
            print("pass")
            passed_tests.append(test_file)
        else:
            print("FAIL")
            failed_tests.append(test_file)
            if not cmd_args.show_output:
                print("### TEST ###")
                print(output_test, end="")
                print("### TRUTH ###")
                print(output_truth, end="")
                print("### DIFF ###")
                print_diff(output_truth, output_test)

        # Print test output metrics, if there are any.
        if output_metrics:
            for metric in output_metrics:
                print(test_file, ": ", metric, sep="")

        if cmd_args.show_output:
            print()

    print("{} tests performed".format(len(skipped_tests) + len(passed_tests) + len(failed_tests)))
    print("{} tests passed".format(len(passed_tests)))

    if skipped_tests:
        print("{} tests skipped: {}".format(len(skipped_tests), " ".join(skipped_tests)))
    if failed_tests:
        print("{} tests failed: {}".format(len(failed_tests), " ".join(failed_tests)))

    return not failed_tests


def main():
    global cmd_args

    cmd_parser = argparse.ArgumentParser(
        description="Run network tests for MicroPython",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    cmd_parser.add_argument(
        "-s", "--show-output", action="store_true", help="show test output after running"
    )
    cmd_parser.add_argument(
        "-t", "--trace-output", action="store_true", help="trace test output while running"
    )
    cmd_parser.add_argument(
        "-i", "--instance", action="append", default=[], help="instance(s) to run the tests on"
    )
    cmd_parser.add_argument(
        "-p",
        "--permutations",
        type=int,
        default=1,
        help="repeat the test with this many permutations of the instance order",
    )
    cmd_parser.epilog = (
        "Supported instance types:\r\n"
        " -i pyb:<port>   physical device (eg. pyboard) on provided repl port.\n"
        " -i micropython  unix micropython instance, path customised with MICROPY_MICROPYTHON env.\n"
        " -i cpython      desktop python3 instance, path customised with MICROPY_CPYTHON3 env.\n"
        " -i exec:<path>  custom program run on provided path.\n"
        "Each instance arg can optionally have custom env provided, eg. <cmd>,ENV=VAR,ENV=VAR...\n"
    )
    cmd_parser.add_argument("files", nargs="+", help="input test files")
    cmd_args = cmd_parser.parse_args()

    # clear search path to make sure tests use only builtin modules and those in extmod
    os.environ["MICROPYPATH"] = os.pathsep.join((".frozen", "../extmod"))

    test_files = prepare_test_file_list(cmd_args.files)
    max_instances = max(t[1] for t in test_files)

    instances_truth = [PyInstanceSubProcess([PYTHON_TRUTH]) for _ in range(max_instances)]

    instances_test = []
    for i in cmd_args.instance:
        # Each instance arg is <cmd>,ENV=VAR,ENV=VAR...
        i = i.split(",")
        cmd = i[0]
        env = i[1:]
        if cmd.startswith("exec:"):
            instances_test.append(PyInstanceSubProcess([cmd[len("exec:") :]], env))
        elif cmd == "micropython":
            instances_test.append(PyInstanceSubProcess([MICROPYTHON], env))
        elif cmd == "cpython":
            instances_test.append(PyInstanceSubProcess([CPYTHON3], env))
        elif cmd.startswith("pyb:"):
            instances_test.append(PyInstancePyboard(cmd[len("pyb:") :]))
        else:
            print("unknown instance string: {}".format(cmd), file=sys.stderr)
            sys.exit(1)

    for _ in range(max_instances - len(instances_test)):
        instances_test.append(PyInstanceSubProcess([MICROPYTHON]))

    all_pass = True
    try:
        for i, instances_test_permutation in enumerate(itertools.permutations(instances_test)):
            if i >= cmd_args.permutations:
                break

            all_pass &= run_tests(test_files, instances_truth, instances_test_permutation)

    finally:
        for i in instances_truth:
            i.close()
        for i in instances_test:
            i.close()

    if not all_pass:
        sys.exit(1)


if __name__ == "__main__":
    main()



# Send L2CAP data as fast as possible and time it.

from micropython import const
import time, machine, bluetooth, random

if not hasattr(bluetooth.BLE, "l2cap_connect"):
    print("SKIP")
    raise SystemExit

TIMEOUT_MS = 1000

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_PERIPHERAL_CONNECT = const(7)
_IRQ_PERIPHERAL_DISCONNECT = const(8)
_IRQ_L2CAP_ACCEPT = const(22)
_IRQ_L2CAP_CONNECT = const(23)
_IRQ_L2CAP_DISCONNECT = const(24)
_IRQ_L2CAP_RECV = const(25)
_IRQ_L2CAP_SEND_READY = const(26)

_L2CAP_PSM = const(22)
_L2CAP_MTU = const(512)

_PAYLOAD_LEN = const(_L2CAP_MTU)
_NUM_PAYLOADS = const(20)

_RANDOM_SEED = 22


waiting_events = {}


def irq(event, data):
    if event == _IRQ_CENTRAL_CONNECT:
        conn_handle, addr_type, addr = data
        waiting_events[event] = conn_handle
    elif event == _IRQ_PERIPHERAL_CONNECT:
        conn_handle, addr_type, addr = data
        waiting_events[event] = conn_handle
    elif event == _IRQ_L2CAP_ACCEPT:
        conn_handle, cid, psm, our_mtu, peer_mtu = data
        waiting_events[event] = (conn_handle, cid, psm)
    elif event == _IRQ_L2CAP_CONNECT:
        conn_handle, cid, psm, our_mtu, peer_mtu = data
        waiting_events[event] = (conn_handle, cid, psm, our_mtu, peer_mtu)

    if event not in waiting_events:
        waiting_events[event] = None


def wait_for_event(event, timeout_ms):
    t0 = time.ticks_ms()
    while time.ticks_diff(time.ticks_ms(), t0) < timeout_ms:
        if event in waiting_events:
            return waiting_events.pop(event)
        machine.idle()
    raise ValueError("Timeout waiting for {}".format(event))


def send_data(ble, conn_handle, cid):
    buf = bytearray(_PAYLOAD_LEN)
    for i in range(_NUM_PAYLOADS):
        for j in range(_PAYLOAD_LEN):
            buf[j] = random.randint(0, 255)
        if not ble.l2cap_send(conn_handle, cid, buf):
            wait_for_event(_IRQ_L2CAP_SEND_READY, TIMEOUT_MS)


def recv_data(ble, conn_handle, cid):
    buf = bytearray(_PAYLOAD_LEN)
    recv_bytes = 0
    recv_correct = 0
    expected_bytes = _PAYLOAD_LEN * _NUM_PAYLOADS
    ticks_first_byte = 0
    while recv_bytes < expected_bytes:
        wait_for_event(_IRQ_L2CAP_RECV, TIMEOUT_MS)
        if not ticks_first_byte:
            ticks_first_byte = time.ticks_ms()
        while True:
            n = ble.l2cap_recvinto(conn_handle, cid, buf)
            if n == 0:
                break
            recv_bytes += n
            for i in range(n):
                if buf[i] == random.randint(0, 255):
                    recv_correct += 1
    ticks_end = time.ticks_ms()
    return recv_bytes, recv_correct, time.ticks_diff(ticks_end, ticks_first_byte)


# Acting in peripheral role.
def instance0():
    multitest.globals(BDADDR=ble.config("mac"))
    ble.gap_advertise(20_000, b"\x02\x01\x06\x04\xffMPY")
    multitest.next()
    try:
        # Wait for central to connect to us.
        conn_handle = wait_for_event(_IRQ_CENTRAL_CONNECT, TIMEOUT_MS)

        ble.l2cap_listen(_L2CAP_PSM, _L2CAP_MTU)

        conn_handle, cid, psm = wait_for_event(_IRQ_L2CAP_ACCEPT, TIMEOUT_MS)
        conn_handle, cid, psm, our_mtu, peer_mtu = wait_for_event(_IRQ_L2CAP_CONNECT, TIMEOUT_MS)

        random.seed(_RANDOM_SEED)

        send_data(ble, conn_handle, cid)

        wait_for_event(_IRQ_L2CAP_DISCONNECT, TIMEOUT_MS)

        # Wait for the central to disconnect.
        wait_for_event(_IRQ_CENTRAL_DISCONNECT, TIMEOUT_MS)
    finally:
        ble.active(0)


# Acting in central role.
def instance1():
    multitest.next()
    try:
        # Connect to peripheral and then disconnect.
        ble.gap_connect(*BDADDR)
        conn_handle = wait_for_event(_IRQ_PERIPHERAL_CONNECT, TIMEOUT_MS)

        ble.l2cap_connect(conn_handle, _L2CAP_PSM, _L2CAP_MTU)
        conn_handle, cid, psm, our_mtu, peer_mtu = wait_for_event(_IRQ_L2CAP_CONNECT, TIMEOUT_MS)

        random.seed(_RANDOM_SEED)

        recv_bytes, recv_correct, total_ticks = recv_data(ble, conn_handle, cid)

        # Disconnect channel.
        ble.l2cap_disconnect(conn_handle, cid)
        wait_for_event(_IRQ_L2CAP_DISCONNECT, TIMEOUT_MS)

        multitest.output_metric(
            "Received {}/{} bytes in {} ms. {} B/s".format(
                recv_bytes, recv_correct, total_ticks, recv_bytes * 1000 // total_ticks
            )
        )

        # Disconnect from peripheral.
        ble.gap_disconnect(conn_handle)
        wait_for_event(_IRQ_PERIPHERAL_DISCONNECT, TIMEOUT_MS)
    finally:
        ble.active(0)


ble = bluetooth.BLE()
ble.active(1)
ble.irq(irq)



els = []
d = {1:2,3:4}
a = d.popitem()
print(len(d))
els.append(a)
a = d.popitem()
print(len(d))
els.append(a)
try:
    print(d.popitem(), "!!!",)
except KeyError:
    print("Raised KeyError")
else:
    print("Did not raise KeyError")
print(sorted(els))




# test thread coordination using a lock object
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

import _thread

lock = _thread.allocate_lock()
n_thread = 10
n_finished = 0


def thread_entry(idx):
    global n_finished
    while True:
        with lock:
            if n_finished == idx:
                break
    print("my turn:", idx)
    with lock:
        n_finished += 1


# spawn threads
for i in range(n_thread):
    _thread.start_new_thread(thread_entry, (i,))

# busy wait for threads to finish
while n_finished < n_thread:
    pass



# Test importing frozen functions.

# A simple bytecode function with no children.
def f():
    return 1


print(__name__, f())


# A simple bytecode generator with no children.
def g():
    yield 2


print(__name__, next(g()))



# Test waiting on a task

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


import time

if hasattr(time, "ticks_ms"):
    ticks = time.ticks_ms
    ticks_diff = time.ticks_diff
else:
    ticks = lambda: int(time.time() * 1000)
    ticks_diff = lambda t1, t0: t1 - t0


async def task(t):
    print("task", t)


async def delay_print(t, s):
    await asyncio.sleep(t)
    print(s)


async def task_raise():
    print("task_raise")
    raise ValueError


async def main():
    print("start")

    # Wait on a task
    t = asyncio.create_task(task(1))
    await t

    # Wait on a task that's already done
    t = asyncio.create_task(task(2))
    await asyncio.sleep(0.001)
    await t

    # Wait again on same task
    await t

    print("----")

    # Create 2 tasks
    ts1 = asyncio.create_task(delay_print(0.2, "hello"))
    ts2 = asyncio.create_task(delay_print(0.4, "world"))

    # Time how long the tasks take to finish, they should execute in parallel
    print("start")
    t0 = ticks()
    await ts1
    t1 = ticks()
    await ts2
    t2 = ticks()
    print("took {} {}".format(round(ticks_diff(t1, t0), -2), round(ticks_diff(t2, t1), -2)))

    # Wait on a task that raises an exception
    t = asyncio.create_task(task_raise())
    try:
        await t
    except ValueError:
        print("ValueError")

    # Wait on a task that raises, but the waiting is done some time later.
    # Need to suppress the "Task exception wasn't retrieved" message.
    asyncio.get_event_loop().set_exception_handler(lambda loop, context: None)
    t = asyncio.create_task(task_raise())
    for _ in range(5):
        await asyncio.sleep(0)
    try:
        await t
    except ValueError:
        print("ValueError")


asyncio.run(main())



import socket, sys


def test_non_existent():
    try:
        res = socket.getaddrinfo("nonexistent.example.com", 80)
        print("getaddrinfo returned", res)
    except OSError as e:
        print("getaddrinfo raised")


def test_bogus():
    try:
        res = socket.getaddrinfo("hey.!!$$", 80)
        print("getaddrinfo returned", res)
    except OSError as e:
        print("getaddrinfo raised")
    except Exception as e:
        print("getaddrinfo raised")  # CPython raises UnicodeError!?


def test_ip_addr():
    try:
        res = socket.getaddrinfo("10.10.10.10", 80)
        print("getaddrinfo returned resolutions")
    except Exception as e:
        print("getaddrinfo raised", e)


def test_0_0_0_0():
    try:
        res = socket.getaddrinfo("0.0.0.0", 80)
        print("getaddrinfo returned resolutions")
    except Exception as e:
        print("getaddrinfo raised", e)


def test_valid():
    try:
        res = socket.getaddrinfo("micropython.org", 80)
        print("getaddrinfo returned resolutions")
    except Exception as e:
        print("getaddrinfo raised", e)


test_funs = [n for n in dir() if n.startswith("test_")]
for f in sorted(test_funs):
    print("--", f, end=": ")
    eval(f + "()")



# MicroPython SSD1306 OLED driver, I2C and SPI interfaces

from micropython import const
import framebuf


# register definitions
SET_CONTRAST = const(0x81)
SET_ENTIRE_ON = const(0xA4)
SET_NORM_INV = const(0xA6)
SET_DISP = const(0xAE)
SET_MEM_ADDR = const(0x20)
SET_COL_ADDR = const(0x21)
SET_PAGE_ADDR = const(0x22)
SET_DISP_START_LINE = const(0x40)
SET_SEG_REMAP = const(0xA0)
SET_MUX_RATIO = const(0xA8)
SET_IREF_SELECT = const(0xAD)
SET_COM_OUT_DIR = const(0xC0)
SET_DISP_OFFSET = const(0xD3)
SET_COM_PIN_CFG = const(0xDA)
SET_DISP_CLK_DIV = const(0xD5)
SET_PRECHARGE = const(0xD9)
SET_VCOM_DESEL = const(0xDB)
SET_CHARGE_PUMP = const(0x8D)


# Subclassing FrameBuffer provides support for graphics primitives
# http://docs.micropython.org/en/latest/pyboard/library/framebuf.html
class SSD1306(framebuf.FrameBuffer):
    def __init__(self, width, height, external_vcc):
        self.width = width
        self.height = height
        self.external_vcc = external_vcc
        self.pages = self.height // 8
        self.buffer = bytearray(self.pages * self.width)
        super().__init__(self.buffer, self.width, self.height, framebuf.MONO_VLSB)
        self.init_display()

    def init_display(self):
        for cmd in (
            SET_DISP,  # display off
            # address setting
            SET_MEM_ADDR,
            0x00,  # horizontal
            # resolution and layout
            SET_DISP_START_LINE,  # start at line 0
            SET_SEG_REMAP | 0x01,  # column addr 127 mapped to SEG0
            SET_MUX_RATIO,
            self.height - 1,
            SET_COM_OUT_DIR | 0x08,  # scan from COM[N] to COM0
            SET_DISP_OFFSET,
            0x00,
            SET_COM_PIN_CFG,
            0x02 if self.width > 2 * self.height else 0x12,
            # timing and driving scheme
            SET_DISP_CLK_DIV,
            0x80,
            SET_PRECHARGE,
            0x22 if self.external_vcc else 0xF1,
            SET_VCOM_DESEL,
            0x30,  # 0.83*Vcc
            # display
            SET_CONTRAST,
            0xFF,  # maximum
            SET_ENTIRE_ON,  # output follows RAM contents
            SET_NORM_INV,  # not inverted
            SET_IREF_SELECT,
            0x30,  # enable internal IREF during display on
            # charge pump
            SET_CHARGE_PUMP,
            0x10 if self.external_vcc else 0x14,
            SET_DISP | 0x01,  # display on
        ):  # on
            self.write_cmd(cmd)
        self.fill(0)
        self.show()

    def poweroff(self):
        self.write_cmd(SET_DISP)

    def poweron(self):
        self.write_cmd(SET_DISP | 0x01)

    def contrast(self, contrast):
        self.write_cmd(SET_CONTRAST)
        self.write_cmd(contrast)

    def invert(self, invert):
        self.write_cmd(SET_NORM_INV | (invert & 1))

    def rotate(self, rotate):
        self.write_cmd(SET_COM_OUT_DIR | ((rotate & 1) << 3))
        self.write_cmd(SET_SEG_REMAP | (rotate & 1))

    def show(self):
        x0 = 0
        x1 = self.width - 1
        if self.width != 128:
            # narrow displays use centred columns
            col_offset = (128 - self.width) // 2
            x0 += col_offset
            x1 += col_offset
        self.write_cmd(SET_COL_ADDR)
        self.write_cmd(x0)
        self.write_cmd(x1)
        self.write_cmd(SET_PAGE_ADDR)
        self.write_cmd(0)
        self.write_cmd(self.pages - 1)
        self.write_data(self.buffer)


class SSD1306_I2C(SSD1306):
    def __init__(self, width, height, i2c, addr=0x3C, external_vcc=False):
        self.i2c = i2c
        self.addr = addr
        self.temp = bytearray(2)
        self.write_list = [b"\x40", None]  # Co=0, D/C#=1
        super().__init__(width, height, external_vcc)

    def write_cmd(self, cmd):
        self.temp[0] = 0x80  # Co=1, D/C#=0
        self.temp[1] = cmd
        self.i2c.writeto(self.addr, self.temp)

    def write_data(self, buf):
        self.write_list[1] = buf
        self.i2c.writevto(self.addr, self.write_list)


class SSD1306_SPI(SSD1306):
    def __init__(self, width, height, spi, dc, res, cs, external_vcc=False):
        self.rate = 10 * 1024 * 1024
        dc.init(dc.OUT, value=0)
        res.init(res.OUT, value=0)
        cs.init(cs.OUT, value=1)
        self.spi = spi
        self.dc = dc
        self.res = res
        self.cs = cs
        import time

        self.res(1)
        time.sleep_ms(1)
        self.res(0)
        time.sleep_ms(10)
        self.res(1)
        super().__init__(width, height, external_vcc)

    def write_cmd(self, cmd):
        self.spi.init(baudrate=self.rate, polarity=0, phase=0)
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf):
        self.spi.init(baudrate=self.rate, polarity=0, phase=0)
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(buf)
        self.cs(1)



metadata(version="0.5.1")

require("re")
require("base64")
require("binascii")
require("functools")
require("string")
# require("calendar") TODO
require("abc")
require("email.errors")
require("email.header")
require("email.charset")
require("email.utils")

package("email")



# Test sys.std*.buffer objects.

import sys

try:
    sys.stdout.buffer
    sys.stdin.buffer
    sys.stderr.buffer
except AttributeError:
    print("SKIP")
    raise SystemExit


# force cpython to flush after every print
# this is to sequence stdout and stderr
def print_flush(*args, **kwargs):
    try:
        print(*args, **kwargs, flush=True)
    except TypeError:
        print(*args, **kwargs)


print_flush("==stdin==")
print_flush(sys.stdin.buffer.fileno())


print_flush("==stdout==")
print_flush(sys.stdout.buffer.fileno())
n_text = sys.stdout.write("The quick brown fox jumps over the lazy dog\n")
sys.stdout.flush()
n_binary = sys.stdout.buffer.write("The quick brown fox jumps over the lazy dog\n".encode("utf-8"))
sys.stdout.buffer.flush()
print_flush("n_text:{} n_binary:{}".format(n_text, n_binary))

# temporarily disabling unicode tests until future PR which fixes unicode write character count
# n_text = sys.stdout.write("🚀")
# sys.stdout.flush()
# n_binary = sys.stdout.buffer.write("🚀".encode("utf-8"))
# sys.stdout.buffer.flush()
# print_flush("")
# print_flush("n_text:{} n_binary:{}".format(n_text, n_binary))
# n_text = sys.stdout.write("1🚀2a3α4b5β6c7γ8d9δ0ぁ1🙐")
# sys.stdout.flush()
# n_binary = sys.stdout.buffer.write("1🚀2a3α4b5β6c7γ8d9δ0ぁ1🙐".encode("utf-8"))
# sys.stdout.buffer.flush()
# print_flush("")
# print_flush("n_text:{} n_binary:{}".format(n_text, n_binary))


print_flush("==stderr==")
print_flush(sys.stderr.buffer.fileno())
n_text = sys.stderr.write("The quick brown fox jumps over the lazy dog\n")
sys.stderr.flush()
n_binary = sys.stderr.buffer.write("The quick brown fox jumps over the lazy dog\n".encode("utf-8"))
sys.stderr.buffer.flush()
print_flush("n_text:{} n_binary:{}".format(n_text, n_binary))

# temporarily disabling unicode tests until future PR which fixes unicode write character count
# n_text = sys.stderr.write("🚀")
# sys.stderr.flush()
# n_binary = sys.stderr.buffer.write("🚀".encode("utf-8"))
# sys.stderr.buffer.flush()
# print_flush("")
# print_flush("n_text:{} n_binary:{}".format(n_text, n_binary))
# print_flush("")
